@@ -17,22 +17,22 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark-editor.c,v 1.34 2004/03/25 04:32:23 makeinu Exp $
+ *  $Id: kz-bookmark-editor.c,v 1.1.1.1 2005/03/27 12:58:28 poincare Exp $
  */
 
 #include "kz-bookmark-editor.h"
 
 #include <string.h>
+#include <glib/gi18n.h>
 
 #include "kazehakase.h"
 #include "gobject-utils.h"
-#include "intl.h"
-#include "egg-toggle-action.h"
 #include "kz-window.h"
 #include "kz-bookmark-edit.h"
 #include "kz-actions-bookmark.h"
 #include "kz-icons.h"
 #include "kz-bookmarks-view.h"
+#include "kz-bookmark-file.h"
 
 
 enum {
@@ -63,7 +63,7 @@ static void     kz_bookmark_editor_set_folder_tree   (KzBookmarkEditor *editor);
 static void     kz_bookmark_editor_set_bookmark_tree (KzBookmarkEditor *editor,
 						      KzBookmark       *parent);
 
-static void     cb_menu_merge_add_widget        (EggMenuMerge     *merge,
+static void     cb_menu_merge_add_widget        (GtkUIManager     *merge,
 						 GtkWidget        *widget,
 						 GtkBox           *box);
 static void     cb_folder_view_cursor_changed   (GtkTreeView      *treeview,
@@ -119,10 +119,10 @@ kz_bookmark_editor_class_init (KzBookmarkEditorClass *klass)
 
 #define SET_SENSITIVE(editor, name, sensitive)				\
 {									\
-	EggAction *action;						\
-	action = egg_action_group_get_action((editor)->action_group,	\
+	GtkAction *action;						\
+	action = gtk_action_group_get_action((editor)->action_group,	\
 					     (name));			\
-	egg_action_set_sensitive(action, (sensitive));			\
+	g_object_set(action, "sensitive", (sensitive), NULL);			\
 }
 
 
@@ -132,9 +132,9 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	GtkWidget *hpaned, *vpaned, *main_vbox;
 	GtkWidget *scrwin[3], *treeview[2], *edit_view, *statusbar;
 	GtkTreeSelection *selection;
-	GtkAccelGroup  *accels;
-	EggActionGroup *actions;
-	EggMenuMerge   *menu_merge;
+	GtkActionGroup *actions;
+	GtkAccelGroup *accel_group;
+	GtkUIManager   *menu_merge;
 	GError *err = NULL;
 
 	g_object_set(G_OBJECT(editor), "allow-shrink", TRUE, NULL);
@@ -144,11 +144,10 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	gtk_window_set_position(GTK_WINDOW(editor),
 				GTK_WIN_POS_CENTER_ON_PARENT);
 
-	accels     = gtk_accel_group_new();
-	actions    = kz_actions_bookmark_create_group(editor, accels);
-	menu_merge = egg_menu_merge_new();
+	menu_merge = gtk_ui_manager_new();
+	accel_group = gtk_ui_manager_get_accel_group(menu_merge);
 
-	gtk_window_add_accel_group(GTK_WINDOW(editor), accels);
+	actions    = kz_actions_bookmark_create_group(editor, accel_group);
 
 	/* toplevel vbox */
 	main_vbox = gtk_vbox_new(FALSE, 0);
@@ -156,11 +155,10 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	gtk_widget_show(main_vbox);
 
 	/* menu and toolbar */
-	egg_menu_merge_set_accel_group(menu_merge, accels);
-	egg_menu_merge_insert_action_group(menu_merge, actions, 0);
+	gtk_ui_manager_insert_action_group(menu_merge, actions, 0);
 	g_signal_connect(menu_merge, "add_widget",
 			 G_CALLBACK(cb_menu_merge_add_widget), main_vbox);
-	egg_menu_merge_add_ui_from_file(menu_merge,
+	gtk_ui_manager_add_ui_from_file(menu_merge,
 					KZ_SYSCONFDIR"/kz-ui-bookmarks.xml",
 					&err);
 	if (err)
@@ -168,8 +166,11 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 		g_warning("%s", err->message);
 		g_error_free(err);
 	}
-        egg_menu_merge_ensure_update(menu_merge);
+        gtk_ui_manager_ensure_update(menu_merge);
 
+	gtk_window_add_accel_group(GTK_WINDOW(editor), 
+				   gtk_ui_manager_get_accel_group(menu_merge));
+	
 	/* paned for main contents */
 	hpaned = gtk_hpaned_new();
 	gtk_box_pack_start(GTK_BOX(main_vbox), hpaned, TRUE, TRUE, 0);
@@ -190,7 +191,7 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	gtk_container_add(GTK_CONTAINER(scrwin[0]), treeview[0]);
 	gtk_widget_show(treeview[0]);
 
-	g_signal_connect(G_OBJECT(treeview[0]), "cursor-changed",
+	g_signal_connect(treeview[0], "cursor-changed",
 			 G_CALLBACK(cb_folder_view_cursor_changed), editor);
 
 	/* vpaned */
@@ -213,12 +214,12 @@ kz_bookmark_editor_init (KzBookmarkEditor *editor)
 	gtk_container_add(GTK_CONTAINER(scrwin[1]), treeview[1]);
 	gtk_widget_show(treeview[1]);
 
-	g_signal_connect(G_OBJECT(treeview[1]), "cursor-changed",
+	g_signal_connect(treeview[1], "cursor-changed",
 			 G_CALLBACK(cb_bookmarks_view_cursor_changed), editor);
-	g_signal_connect(G_OBJECT(selection), "changed",
+	g_signal_connect(selection, "changed",
 			 G_CALLBACK(cb_bookmarks_view_selection_changed),
 			 editor);
-	g_signal_connect(G_OBJECT(treeview[1]), "button-press-event",
+	g_signal_connect(treeview[1], "button-press-event",
 			 G_CALLBACK(cb_bookmarks_view_button_press),
 			 editor);
 
@@ -339,8 +340,6 @@ kz_bookmark_editor_delete_event (GtkWidget *widget, GdkEventAny *event)
 {
 	KzBookmarkEditor *editor;
 
-	g_return_val_if_fail(KZ_IS_BOOKMARK_EDITOR(widget), FALSE);
-
 	editor = KZ_BOOKMARK_EDITOR(widget);
 	kz_bookmark_editor_store_state(editor);
 
@@ -385,10 +384,10 @@ kz_bookmark_editor_set_bookmark_tree (KzBookmarkEditor *editor,
 		SET_SENSITIVE(editor, "GoUp", TRUE);
 	}
 
-	if (parent && !kz_bookmark_is_file(parent))
+	if (parent && !KZ_IS_BOOKMARK_FILE(parent))
 		parent = kz_bookmark_get_parent_file(parent);
 
-	if (parent && kz_bookmark_file_is_editable(parent))
+	if (parent && kz_bookmark_file_is_editable(KZ_BOOKMARK_FILE(parent)))
 		editable = TRUE;
 	else
 		editable = FALSE;
@@ -396,6 +395,7 @@ kz_bookmark_editor_set_bookmark_tree (KzBookmarkEditor *editor,
 	SET_SENSITIVE(editor, "InsertBookmark",       editable);
 	SET_SENSITIVE(editor, "InsertFolder",         editable);
 	SET_SENSITIVE(editor, "InsertRemoteBookmark", editable);
+	SET_SENSITIVE(editor, "InsertSmartBookmark",  editable);
 	SET_SENSITIVE(editor, "InsertSeparator",      editable);
 }
 
@@ -438,7 +438,7 @@ kz_bookmark_editor_set_tree_mode (KzBookmarkEditor *editor,
 void
 kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 {
-	EggAction *action;
+	GtkAction *action;
 	gint client_x, client_y, width, height;
 	gboolean show_folder_view, show_edit_view;
 
@@ -448,15 +448,15 @@ kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 				 &client_x, &client_y,
 				 &width, &height, NULL);
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideFolderView");
 	show_folder_view
-		= egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action));
+		= gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideContentView");
 	show_edit_view
-		= egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action));
+		= gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action));
 
 	KZ_CONF_SET("BookmarkEditorWindow", "width",  width,  INT);
 	KZ_CONF_SET("BookmarkEditorWindow", "height", height, INT);
@@ -479,9 +479,9 @@ kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 	}
 
 	/* mode */
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "TreeMode");
-	if (egg_toggle_action_get_active(EGG_TOGGLE_ACTION(action)))
+	if (gtk_toggle_action_get_active(GTK_TOGGLE_ACTION(action)))
 		KZ_CONF_SET_STR("BookmarkEditorWindow", "mode",
 				"TreeMode");
 	else
@@ -493,7 +493,7 @@ kz_bookmark_editor_store_state (KzBookmarkEditor *editor)
 void
 kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 {
-	EggAction *action;
+	GtkAction *action;
 	gint width = 600, height = 450;
 	gint folder_view_width = 150, bookmarks_view_height = 230;
 	gboolean show_folder_view = TRUE, show_edit_view = FALSE;
@@ -516,14 +516,14 @@ kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 	/* size */
 	gtk_window_set_default_size(GTK_WINDOW(editor), width, height);
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideFolderView");
-	egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action),
+	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action),
 				     show_folder_view);
 
-	action = egg_action_group_get_action(editor->action_group,
+	action = gtk_action_group_get_action(editor->action_group,
 					     "ShowHideContentView");
-	egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action),
+	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action),
 				     show_edit_view);
 
 	gtk_widget_set_size_request(GTK_WIDGET(editor->scrolledwin[0]),
@@ -534,15 +534,15 @@ kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 	/* mode */
 	if (mode && !strcmp(mode, "TreeMode"))
 	{
-		action = egg_action_group_get_action(editor->action_group,
+		action = gtk_action_group_get_action(editor->action_group,
 						     "TreeMode");
-		egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action), TRUE);
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
 	}
 	else
 	{
-		action = egg_action_group_get_action(editor->action_group,
+		action = gtk_action_group_get_action(editor->action_group,
 						     "ListMode");
-		egg_toggle_action_set_active(EGG_TOGGLE_ACTION(action), TRUE);
+		gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), TRUE);
 	}
 
 	g_free(mode);
@@ -550,7 +550,7 @@ kz_bookmark_editor_restore_state (KzBookmarkEditor *editor)
 
 
 static void
-cb_menu_merge_add_widget (EggMenuMerge *merge, GtkWidget *widget,
+cb_menu_merge_add_widget (GtkUIManager *merge, GtkWidget *widget,
 			  GtkBox *box)
 {
 	gtk_box_pack_start (box, widget, FALSE, FALSE, 0);
@@ -566,7 +566,6 @@ cb_folder_view_cursor_changed (GtkTreeView *treeview, KzBookmarkEditor *editor)
 	GtkTreePath *path = NULL;
 	KzBookmark *folder;
 
-	g_return_if_fail(treeview);
 	g_return_if_fail(KZ_IS_BOOKMARK_EDITOR(editor));
 
 	kz_bookmark_edit_clear(KZ_BOOKMARK_EDIT(editor->content_view));
@@ -599,7 +598,6 @@ cb_bookmarks_view_cursor_changed (GtkTreeView *treeview,
 	GtkTreePath *path = NULL;
 	KzBookmark *bookmark = NULL;
 
-	g_return_if_fail(GTK_IS_TREE_VIEW(treeview));
 	g_return_if_fail(KZ_IS_BOOKMARK_EDITOR(editor));
 
 	model = gtk_tree_view_get_model(treeview);
@@ -630,7 +628,6 @@ cb_bookmarks_view_selection_changed (GtkTreeSelection *selection,
 	GtkTreeIter iter;
 	gboolean selected;
 
-	g_return_if_fail(GTK_IS_TREE_SELECTION(selection));
 	g_return_if_fail(KZ_IS_BOOKMARK_EDITOR(editor));
 
 	selected = gtk_tree_selection_get_selected(selection, &model, &iter);
