@@ -18,38 +18,26 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-bookmark.c,v 1.34 2004/03/24 14:45:52 ikezoe Exp $
+ *  $Id: kz-bookmark.c,v 1.1.1.1 2005/03/27 12:58:27 poincare Exp $
  */
 
 #include "kz-bookmark.h"
 
 #include <string.h>
+#include <gtk/gtk.h>
+#include <glib/gi18n.h>
 #include "gobject-utils.h"
-#include "intl.h"
-#include "egg-marshalers.h"
-
-#include "kz-downloader.h"
-#include "kz-io.h"
-
-/* bookmark file types */
-#include "kz-xbel.h"
-#include "kz-rss.h"
-#include "kz-nsbookmark.h"
-#include "kz-lirs.h"
-#include "kz-hinadi.h"
-#include "kz-google.h"
-#include "kz-scrap-bookmark.h"
+#include "glib-utils.h"
+#include "kz-marshalers.h"
+#include "utils.h"
+#include "eggregex.h"
+#include "kz-bookmark-file.h"
 
 enum {
 	INSERT_CHILD_SIGNAL,
 	REMOVE_CHILD_SIGNAL,
 	MOVE_CHILD_SIGNAL,
 	CHILDREN_REORDERED_SIGNAL,
-	LOAD_START_SIGNAL,
-	LOAD_COMPLETED_SIGNAL,
-	SAVE_START_SIGNAL,
-	SAVE_COMPLETED_SIGNAL,
-	ERROR_SIGNAL,
 	LAST_SIGNAL
 };
 
@@ -58,22 +46,18 @@ enum {
 	PROP_TYPE,
 	PROP_TITLE,
 	PROP_DOC_TITLE,
+	PROP_ID,
 	PROP_LINK,
-	PROP_BOOKMARK_LOCATION,
-	PROP_FILE_TYPE,
 	PROP_DESCRIPTION,
 	PROP_LAST_MODIFIED,
+	PROP_LAST_VISITED,
+	PROP_ADDED_TIME,
 	PROP_INTERVAL,
-	PROP_EDITABLE
+	PROP_CURRENT_POS,
+	PROP_LOCK,
+	PROP_AUTO_REFRESH
 };
 
-#define ADD_BOOKMARK_FILE_TYPE(func) 					\
-{									\
-	KzBookmarkFileType *type;					\
-	gint i;								\
-	for (i = 0; (type = func(i)); i++)				\
-		file_types = g_list_append(file_types, type);		\
-}
 
 static void kz_bookmark_class_init   (KzBookmarkClass *klass);
 static void kz_bookmark_init         (KzBookmark      *bookmark);
@@ -93,41 +77,22 @@ static void kz_bookmark_real_insert_child (KzBookmark *bookmark,
 static void kz_bookmark_real_remove_child (KzBookmark *bookmark,
 					   KzBookmark *child);
 
-static void kz_bookmark_set_state          (KzBookmark *bookmark,
-					    KzBookmarkState state);
-static void kz_bookmark_set_editable       (KzBookmark *bookmark,
-					    gboolean    editable);
-
-/* load/save */
-static void kz_bookmark_load_completed   (KzBookmark   *bookmark,
-					  KzDownloader *dl,
-					  gboolean      error);
-static KzBookmarkFileType *
-            kz_bookmark_detect_file_type (KzBookmark   *bookmark,
-					  gchar        *buf);
-static void cb_downloader_load_complete  (KzDownloader *dl,
-					  KzBookmark   *bookmark);
-static void cb_downloader_load_error     (KzDownloader *dl,
-					  KzBookmark   *bookmark);
-
 static GObjectClass *parent_class = NULL;
 static gint kz_bookmark_signals[LAST_SIGNAL] = {0};
 
-static GList *file_types = NULL;
-
+static GQuark id_quark                = 0;
 static GQuark title_quark             = 0;
 static GQuark doc_title_quark         = 0;
 static GQuark link_quark              = 0;
-static GQuark location_quark          = 0;
-static GQuark file_type_quark         = 0;
 static GQuark description_quark       = 0;
 static GQuark last_mod_quark          = 0;
+static GQuark last_visited_quark      = 0;
+static GQuark added_time_quark        = 0;
 static GQuark parent_quark            = 0;
 static GQuark children_quark          = 0;
-static GQuark interval_quark          = 0;
-static GQuark timer_quark             = 0;
-static GQuark state_quark             = 0;
-
+static GQuark current_pos_quark       = 0;
+static GQuark lock_quark              = 0;
+static GQuark auto_refresh_quark      = 0;
 
 GType
 kz_bookmark_type_get_type (void)
@@ -158,17 +123,6 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 {
 	GObjectClass *object_class;
 
-	if (!file_types)
-	{
-		ADD_BOOKMARK_FILE_TYPE(kz_rss_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_lirs_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_hinadi_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_google_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_xbel_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_nsbookmark_get_file_types);
-		ADD_BOOKMARK_FILE_TYPE(kz_scrap_bookmark_get_file_types);
-	}
-
 	parent_class = g_type_class_peek_parent (klass);
 	object_class = (GObjectClass *) klass;
 
@@ -180,10 +134,6 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 	klass->remove_child       = kz_bookmark_real_remove_child;
 	klass->move_child         = NULL; /* kz_bookmark_real_move_child; */
 	klass->children_reordered = NULL;
-	klass->load_start         = NULL;
-	klass->load_completed     = NULL;
-	klass->save_start         = NULL;
-	klass->save_completed     = NULL;
 
 	g_object_class_install_property(
 		object_class,
@@ -196,6 +146,16 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 			KZ_BOOKMARK_NORMAL,
 			G_PARAM_READWRITE));
 
+	g_object_class_install_property(
+		object_class,
+		 PROP_ID,
+		 g_param_spec_string(
+			 "id",
+			 _("ID"),
+			 _("The ID of the bookmark"),
+			 NULL,
+			 G_PARAM_READWRITE));
+
 	g_object_class_install_property(
 		object_class,
 		 PROP_TITLE,
@@ -226,26 +186,6 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 			 NULL,
 			 G_PARAM_READWRITE));
 
-	g_object_class_install_property(
-		object_class,
-		 PROP_BOOKMARK_LOCATION,
-		 g_param_spec_string(
-			 "location",
-			 _("Location of bookmark file itself"),
-			 _("The location of the bookmark file itself"),
-			 NULL,
-			 G_PARAM_READWRITE));
-
-	g_object_class_install_property(
-		object_class,
-		 PROP_FILE_TYPE,
-		 g_param_spec_string(
-			 "file-type",
-			 _("File type"),
-			 _("File type of the bookmark file"),
-			 NULL,
-			 G_PARAM_READWRITE));
-
 	g_object_class_install_property(
 		object_class,
 		 PROP_DESCRIPTION,
@@ -270,11 +210,11 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 
 	g_object_class_install_property(
 		object_class,
-		 PROP_INTERVAL,
+		 PROP_LAST_VISITED,
 		 g_param_spec_uint(
-			 "interval",
-			 _("Interval"),
-			 _("Update interval"),
+			 "last-visited",
+			 _("Last Visited Time"),
+			 _("The time of the user's last visit to the link"),
 			 0,
 			 G_MAXUINT,
 			 0,
@@ -282,12 +222,46 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 
 	g_object_class_install_property(
 		object_class,
-		 PROP_EDITABLE,
-		 g_param_spec_boolean(
-			 "editable",
-			 _("Editable"),
-			 _("Whether attributes of the bookmark is writable or not"),
-			 TRUE,
+		 PROP_ADDED_TIME,
+		 g_param_spec_uint(
+			 "added-time",
+			 _("Added Time"),
+			 _("The time of the added the bookmark"),
+			 0,
+			 G_MAXUINT,
+			 0,
+			 G_PARAM_READWRITE));
+
+	g_object_class_install_property(
+		object_class,
+		 PROP_CURRENT_POS,
+		 g_param_spec_uint(
+			 "current",
+			 _("Current Position"),
+			 _("Current position in history"),
+			 0,
+			 G_MAXUINT,
+			 0,
+			 G_PARAM_READWRITE));
+
+	g_object_class_install_property(
+		object_class,
+		PROP_LOCK,
+		g_param_spec_boolean(
+			"lock",
+			_("Lock state"),
+			_("Lock state for tab"),
+			FALSE,
+			 G_PARAM_READWRITE));
+
+	g_object_class_install_property(
+		object_class,
+		PROP_AUTO_REFRESH,
+		g_param_spec_boolean(
+			 "auto-refresh",
+			 _("Auto Refresh state"),
+			 _("Auto Refresh state for tab"),
+			 FALSE,
 			 G_PARAM_READWRITE));
 
 	kz_bookmark_signals[INSERT_CHILD_SIGNAL]
@@ -296,7 +270,7 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (KzBookmarkClass, insert_child),
 				NULL, NULL,
-				_egg_marshal_VOID__OBJECT_OBJECT,
+				_kz_marshal_VOID__OBJECT_OBJECT,
 				G_TYPE_NONE, 2,
 				KZ_TYPE_BOOKMARK, KZ_TYPE_BOOKMARK);
 
@@ -316,7 +290,7 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 				G_SIGNAL_RUN_LAST,
 				G_STRUCT_OFFSET (KzBookmarkClass, move_child),
 				NULL, NULL,
-				_egg_marshal_VOID__OBJECT_OBJECT,
+				_kz_marshal_VOID__OBJECT_OBJECT,
 				G_TYPE_NONE, 2,
 				KZ_TYPE_BOOKMARK, KZ_TYPE_BOOKMARK);
 
@@ -330,56 +304,6 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 				g_cclosure_marshal_VOID__VOID,
 				G_TYPE_NONE, 0);
 
-	kz_bookmark_signals[LOAD_START_SIGNAL]
-		= g_signal_new("load_start",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkClass,
-						load_start),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
-
-	kz_bookmark_signals[LOAD_COMPLETED_SIGNAL]
-		= g_signal_new("load_completed",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkClass,
-						load_completed),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
-
-	kz_bookmark_signals[SAVE_START_SIGNAL]
-		= g_signal_new("save_start",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkClass,
-						save_start),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
-
-	kz_bookmark_signals[SAVE_COMPLETED_SIGNAL]
-		= g_signal_new("save_completed",
-			       G_TYPE_FROM_CLASS (klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkClass,
-						save_completed),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__VOID,
-			       G_TYPE_NONE, 0);
-
-	kz_bookmark_signals[ERROR_SIGNAL]
-		= g_signal_new("error",
-			       G_TYPE_FROM_CLASS(klass),
-			       G_SIGNAL_RUN_LAST,
-			       G_STRUCT_OFFSET (KzBookmarkClass,
-						error),
-			       NULL, NULL,
-			       g_cclosure_marshal_VOID__STRING,
-			       G_TYPE_NONE, 1, G_TYPE_STRING);
-
 	/* FIXME! add other properties */
 	/*
 	 * candidates:
@@ -390,26 +314,24 @@ kz_bookmark_class_init (KzBookmarkClass *klass)
 	 *   * ......
 	 */
 
-	title_quark       = g_quark_from_string("KzBookmark::Title");
-	doc_title_quark   = g_quark_from_string("KzBookmark::DocumentTitle");
-	link_quark        = g_quark_from_string("KzBookmark::Link");
-	location_quark    = g_quark_from_string("KzBookmark::BookmarkLocation");
-	file_type_quark   = g_quark_from_string("KzBookmark::FileType");
-	description_quark = g_quark_from_string("KzBookmark::Description");
-	last_mod_quark    = g_quark_from_string("KzBookmark::LastModified");
-	parent_quark      = g_quark_from_string("KzBookmark::Parent");
-	children_quark    = g_quark_from_string("KzBookmark::Children");
-	interval_quark    = g_quark_from_string("KzBookmark::Interval");
-	timer_quark       = g_quark_from_string("KzBookmark::Timer");
-	state_quark       = g_quark_from_string("KzBookmark::State");
+	id_quark           = g_quark_from_string("KzBookmark::ID");
+	title_quark        = g_quark_from_string("KzBookmark::Title");
+	doc_title_quark    = g_quark_from_string("KzBookmark::DocumentTitle");
+	link_quark         = g_quark_from_string("KzBookmark::Link");
+	description_quark  = g_quark_from_string("KzBookmark::Description");
+	last_mod_quark     = g_quark_from_string("KzBookmark::LastModified");
+	last_visited_quark = g_quark_from_string("KzBookmark::LastVisited");
+	added_time_quark   = g_quark_from_string("KzBookmark::AddedTime");
+	parent_quark       = g_quark_from_string("KzBookmark::Parent");
+	children_quark     = g_quark_from_string("KzBookmark::Children");
+	current_pos_quark  = g_quark_from_string("KzBookmark::Current");
+	lock_quark         = g_quark_from_string("KzBookmark::Lock");
+	auto_refresh_quark = g_quark_from_string("KzBookmark::AutoRefresh");
 }
 
-
 static void
 kz_bookmark_init (KzBookmark *bookmark)
 {
-	bookmark->type  = KZ_BOOKMARK_NORMAL;
-	bookmark->flags = KZ_BOOKMARK_EDITABLE_FLAG;
 }
 
 
@@ -417,20 +339,11 @@ static void
 kz_bookmark_dispose (GObject *object)
 {
 	KzBookmark *bookmark;
-	guint timer_id;
-
-	g_return_if_fail(KZ_IS_BOOKMARK(object));
-
+	
 	bookmark = KZ_BOOKMARK(object);
 
 	kz_bookmark_remove_all(bookmark);
 
-	timer_id = GPOINTER_TO_UINT(g_object_get_qdata(object, timer_quark));
-	if (timer_id)
-		g_source_remove(timer_id);
-	timer_id = 0;
-	g_object_set_qdata(object, timer_quark, GUINT_TO_POINTER(timer_id));
-
 	if (G_OBJECT_CLASS (parent_class)->dispose)
 		G_OBJECT_CLASS (parent_class)->dispose(object);
 }
@@ -442,7 +355,6 @@ kz_bookmark_dispose (GObject *object)
 				(GDestroyNotify) g_free); \
 }
 
-
 static void
 kz_bookmark_set_property (GObject *object,
 			  guint prop_id,
@@ -455,6 +367,10 @@ kz_bookmark_set_property (GObject *object,
 	case PROP_TYPE:
 		bookmark->type = g_value_get_enum(value);
 		break;
+	case PROP_ID:
+		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
+		CHANGE_STR(object, id_quark, g_value_dup_string(value));
+		break;
 	case PROP_TITLE:
 		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
 		CHANGE_STR(object, title_quark, g_value_dup_string(value));
@@ -468,39 +384,6 @@ kz_bookmark_set_property (GObject *object,
 		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
 		CHANGE_STR(object, link_quark, g_value_dup_string(value));
 		break;
-	case PROP_BOOKMARK_LOCATION:
-	{
-		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
-		CHANGE_STR(object, location_quark,
-			   g_value_dup_string(value));
-		break;
-	}
-	case PROP_FILE_TYPE:
-	{
-		gchar *str = g_value_dup_string(value);
-		KzBookmarkFileType *type;
-
-		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
-		g_return_if_fail(!kz_bookmark_get_file_type(bookmark));
-
-		CHANGE_STR(object, file_type_quark, str);
-
-		if (str && *str)
-		{
-			type = kz_bookmark_detect_file_type(bookmark, NULL);
-			if (type && type->init)
-				type->init(bookmark);
-			if (type && !type->to_string)
-			{
-				kz_bookmark_set_editable(bookmark, FALSE);
-				/*
-				 * FIXME!!
-				 * Also set all children as non-editable.
-				 */
-			}
-		}
-		break;
-	}
 	case PROP_DESCRIPTION:
 		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
 		CHANGE_STR(object, description_quark,
@@ -511,35 +394,30 @@ kz_bookmark_set_property (GObject *object,
 		g_object_set_qdata(object, last_mod_quark,
 				   GUINT_TO_POINTER(g_value_get_uint(value)));
 		break;
-	case PROP_INTERVAL:
-	{
-		guint interval, timer_id;
-
-		interval = g_value_get_uint(value);
-		timer_id = GPOINTER_TO_UINT
-				(g_object_get_qdata(object, timer_quark));
-
-		if (timer_id)
-			g_source_remove(timer_id);
-		timer_id = 0;
-
-		if (interval > 0)
-			timer_id = g_timeout_add(60000 * interval,
-						 (gpointer) kz_bookmark_load_in_memory_start,
-						 bookmark);
-
-		g_object_set_qdata(object, interval_quark,
-				   GUINT_TO_POINTER(interval));
-		g_object_set_qdata(object, timer_quark,
-				   GUINT_TO_POINTER(timer_id));
-		break;
-	}
-	case PROP_EDITABLE:
+	case PROP_LAST_VISITED:
 		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
-		if (g_value_get_boolean(value))
-			bookmark->flags |= KZ_BOOKMARK_EDITABLE_FLAG;
-		else
-			bookmark->flags &= ~KZ_BOOKMARK_EDITABLE_FLAG;
+		g_object_set_qdata(object, last_visited_quark,
+				   GUINT_TO_POINTER(g_value_get_uint(value)));
+		break;
+	case PROP_ADDED_TIME:
+		g_return_if_fail(!kz_bookmark_is_separator(bookmark));
+		g_object_set_qdata(object, added_time_quark,
+				   GUINT_TO_POINTER(g_value_get_uint(value)));
+		break;
+	case PROP_CURRENT_POS:
+		g_return_if_fail(kz_bookmark_is_folder(bookmark));
+		g_object_set_qdata(object, current_pos_quark,
+				   GUINT_TO_POINTER(g_value_get_uint(value)));
+		break;
+	case PROP_LOCK:
+		g_return_if_fail(kz_bookmark_is_folder(bookmark));
+		g_object_set_qdata(object, lock_quark,
+				   GINT_TO_POINTER(g_value_get_boolean(value)));
+		break;
+	case PROP_AUTO_REFRESH:
+		g_return_if_fail(kz_bookmark_is_folder(bookmark));
+		g_object_set_qdata(object, auto_refresh_quark,
+				   GINT_TO_POINTER(g_value_get_boolean(value)));
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -562,6 +440,10 @@ kz_bookmark_get_property (GObject *object,
 	case PROP_TYPE:
 		g_value_set_enum(value, bookmark->type);
 		break;
+	case PROP_ID:
+		str = g_object_get_qdata(object, id_quark);
+		g_value_set_string(value, str);
+		break;
 	case PROP_TITLE:
 		str = g_object_get_qdata(object, title_quark);
 		g_value_set_string(value, str);
@@ -574,14 +456,6 @@ kz_bookmark_get_property (GObject *object,
 		str = g_object_get_qdata(object, link_quark);
 		g_value_set_string(value, str);
 		break;
-	case PROP_BOOKMARK_LOCATION:
-		str = g_object_get_qdata(object, location_quark);
-		g_value_set_string(value, str);
-		break;
-	case PROP_FILE_TYPE:
-		str = g_object_get_qdata(object, file_type_quark);
-		g_value_set_string(value, str);
-		break;
 	case PROP_DESCRIPTION:
 		str = g_object_get_qdata(object, description_quark);
 		g_value_set_string(value, str);
@@ -591,20 +465,30 @@ kz_bookmark_get_property (GObject *object,
 							   last_mod_quark));
 		g_value_set_uint(value, time);
 		break;
-	case PROP_INTERVAL:
-	{
-		guint interval;
-
-		interval = GPOINTER_TO_UINT
-				(g_object_get_qdata(object, interval_quark));
-		g_value_set_uint(value, interval);
+	case PROP_LAST_VISITED:
+		time = GPOINTER_TO_UINT(g_object_get_qdata(object,
+							   last_visited_quark));
+		g_value_set_uint(value, time);
 		break;
-	}
-	case PROP_EDITABLE:
-		if (bookmark->flags & KZ_BOOKMARK_EDITABLE_FLAG)
-			g_value_set_boolean(value, TRUE);
-		else
-			g_value_set_boolean(value, FALSE);
+	case PROP_ADDED_TIME:
+		time = GPOINTER_TO_UINT(g_object_get_qdata(object,
+							   added_time_quark));
+		g_value_set_uint(value, time);
+		break;
+	case PROP_CURRENT_POS:
+		time = GPOINTER_TO_UINT(g_object_get_qdata(object,
+							   current_pos_quark));
+		g_value_set_uint(value, time);
+		break;
+	case PROP_LOCK:
+		time = GPOINTER_TO_INT(g_object_get_qdata(object,
+							  lock_quark));
+		g_value_set_boolean(value, time);
+		break;
+	case PROP_AUTO_REFRESH:
+		time = GPOINTER_TO_INT(g_object_get_qdata(object,
+							  auto_refresh_quark));
+		g_value_set_boolean(value, time);
 		break;
 	default:
 		G_OBJECT_WARN_INVALID_PROPERTY_ID(object, prop_id, pspec);
@@ -616,8 +500,8 @@ kz_bookmark_get_property (GObject *object,
 KzBookmark *
 kz_bookmark_new (void)
 {
-	KzBookmark *bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
-							NULL));
+	KzBookmark *bookmark = g_object_new(KZ_TYPE_BOOKMARK,
+					    NULL);
 	return bookmark;
 }
 
@@ -627,9 +511,9 @@ kz_bookmark_pure_folder_new (void)
 {
 	KzBookmark *bookmark;
 
-	bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
-					    "type", KZ_BOOKMARK_PURE_FOLDER,
-					    NULL));
+	bookmark = g_object_new(KZ_TYPE_BOOKMARK,
+			        "type", KZ_BOOKMARK_PURE_FOLDER,
+			        NULL);
 	return bookmark;
 }
 
@@ -639,10 +523,10 @@ kz_bookmark_separator_new (void)
 {
 	KzBookmark *bookmark;
 
-	bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
-					    "title", "-----",
-					    "type",  KZ_BOOKMARK_SEPARATOR,
-					    NULL));
+	bookmark = g_object_new(KZ_TYPE_BOOKMARK,
+			        "title", "-----",
+			        "type",  KZ_BOOKMARK_SEPARATOR,
+				NULL);
 	return bookmark;
 }
 
@@ -654,69 +538,20 @@ kz_bookmark_new_with_attrs (const gchar *title,
 {
 	KzBookmark *bookmark;
 
-	bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
-					    "title",       title,
-					    "link",        uri,
-					    "description", description,
-					    NULL));
+	bookmark = g_object_new(KZ_TYPE_BOOKMARK,
+			        "title",       title,
+				"link",        uri,
+				"description", description,
+				NULL);
 	return bookmark;
 }
 
 
-KzBookmark *
-kz_bookmark_file_new (const gchar *location,
-		      const gchar *title,
-		      const gchar *file_type)
+const gchar *
+kz_bookmark_get_id (KzBookmark *bookmark)
 {
-	KzBookmark *bookmark;
-
-	location = location ? location : "";
-
-	bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
-					    "type",      KZ_BOOKMARK_FOLDER,
-					    "location",  location,
-					    "title",     title,
-					    "file-type", file_type,
-					    NULL));
-	bookmark->flags |= KZ_BOOKMARK_FILE_FLAG;
-	/* kz_bookmark_load_start(bookmark); */
-
-	return bookmark;
-}
-
-
-KzBookmark *
-kz_bookmark_file_create_new (const gchar *location,
-			     const gchar *title,
-			     const gchar *file_type)
-{
-	KzBookmark *bookmark;
-	KzBookmarkFileType *type;
-
-	if (!location)
-	{
-		g_warning(_("kz_bookmark_file_create_new(): "
-			    "location is not specified!"));
-		location = "";
-	}
-
-	bookmark = KZ_BOOKMARK(g_object_new(KZ_TYPE_BOOKMARK,
-					    "type",      KZ_BOOKMARK_FOLDER,
-					    "location",  location,
-					    "title",     title,
-					    "file-type", file_type,
-					    NULL));
-	bookmark->flags |= KZ_BOOKMARK_FILE_FLAG;
-
-	type = kz_bookmark_detect_file_type(bookmark, NULL);
-	if (type && type->from_string)
-	{
-		if (!kz_bookmark_get_file_type(bookmark))
-			kz_bookmark_set_file_type(bookmark, type->file_type);
-		type->from_string(bookmark, NULL, 0, NULL);
-	}
-
-	return bookmark;
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
+	return g_object_get_qdata(G_OBJECT(bookmark), id_quark);
 }
 
 
@@ -752,22 +587,6 @@ kz_bookmark_get_description (KzBookmark *bookmark)
 }
 
 
-const gchar *
-kz_bookmark_get_location (KzBookmark *bookmark)
-{
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
-	return g_object_get_qdata(G_OBJECT(bookmark), location_quark);
-}
-
-
-const gchar *
-kz_bookmark_get_file_type (KzBookmark *bookmark)
-{
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
-	return g_object_get_qdata(G_OBJECT(bookmark), file_type_quark);
-}
-
-
 guint
 kz_bookmark_get_last_modified (KzBookmark *bookmark)
 {
@@ -776,13 +595,20 @@ kz_bookmark_get_last_modified (KzBookmark *bookmark)
 						   last_mod_quark));
 }
 
-
 guint
-kz_bookmark_get_interval (KzBookmark *bookmark)
+kz_bookmark_get_last_visited (KzBookmark *bookmark)
 {
 	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), 0);
 	return GPOINTER_TO_UINT(g_object_get_qdata(G_OBJECT(bookmark),
-						   interval_quark));
+						   last_visited_quark));
+}
+
+guint
+kz_bookmark_get_added_time (KzBookmark *bookmark)
+{
+	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), 0);
+	return GPOINTER_TO_UINT(g_object_get_qdata(G_OBJECT(bookmark),
+						   added_time_quark));
 }
 
 
@@ -795,40 +621,21 @@ kz_bookmark_is_editable (KzBookmark *bookmark)
 
 	parent_file = kz_bookmark_get_parent_file(bookmark);
 
-	if (!KZ_IS_BOOKMARK(parent_file))
-		return FALSE;
+	if (!parent_file) return FALSE;
 
-	if (!kz_bookmark_file_is_editable(parent_file))
+	if (!kz_bookmark_file_is_editable(KZ_BOOKMARK_FILE(parent_file)))
 		return FALSE;
 
 	return TRUE;
 }
 
 
-gboolean
-kz_bookmark_file_is_editable (KzBookmark *bookmark_file)
+void
+kz_bookmark_set_id (KzBookmark *bookmark,
+		    const gchar *id)
 {
-	KzBookmarkFileType *type;
-
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark_file), FALSE);
-	g_return_val_if_fail(kz_bookmark_is_file(bookmark_file), FALSE);
-
-	if (!kz_bookmark_get_file_type(bookmark_file))
-		return FALSE;
-
-	type = kz_bookmark_detect_file_type(bookmark_file, NULL);
-	if (type && type->to_string)
-		return TRUE;
-
-	return FALSE;
-}
-
-
-gboolean
-kz_bookmark_is_file (KzBookmark *bookmark)
-{
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
-	return bookmark->flags & KZ_BOOKMARK_FILE_FLAG ? TRUE : FALSE;
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_object_set(G_OBJECT(bookmark), "id", id, NULL);
 }
 
 
@@ -868,24 +675,6 @@ kz_bookmark_set_description (KzBookmark *bookmark,
 }
 
 
-void
-kz_bookmark_set_location (KzBookmark *bookmark,
-			  const gchar *location)
-{
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-	g_object_set(G_OBJECT(bookmark), "location", location, NULL);
-}
-
-
-void
-kz_bookmark_set_file_type (KzBookmark *bookmark,
-			   const gchar *file_type)
-{
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-	g_object_set(G_OBJECT(bookmark), "file-type", file_type, NULL);
-}
-
-
 void
 kz_bookmark_set_last_modified (KzBookmark *bookmark,
 			       guint time)
@@ -894,14 +683,21 @@ kz_bookmark_set_last_modified (KzBookmark *bookmark,
 	g_object_set(G_OBJECT(bookmark), "last-modified", time, NULL);
 }
 
-
 void
-kz_bookmark_set_interval (KzBookmark *bookmark, guint interval)
+kz_bookmark_set_last_visited (KzBookmark *bookmark,
+			      guint time)
 {
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-	g_object_set(G_OBJECT(bookmark), "interval", interval, NULL);
+	g_object_set(G_OBJECT(bookmark), "last-visited", time, NULL);
 }
 
+void
+kz_bookmark_set_added_time (KzBookmark *bookmark,
+		            guint time)
+{
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+	g_object_set(G_OBJECT(bookmark), "added-time", time, NULL);
+}
 
 gboolean
 kz_bookmark_is_separator (KzBookmark *bookmark)
@@ -915,14 +711,6 @@ kz_bookmark_is_separator (KzBookmark *bookmark)
 }
 
 
-static void
-kz_bookmark_set_editable (KzBookmark *bookmark, gboolean editable)
-{
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-	g_object_set(G_OBJECT(bookmark), "editable", editable, NULL);
-}
-
-
 gboolean
 kz_bookmark_is_folder(KzBookmark *bookmark)
 {
@@ -1006,7 +794,7 @@ kz_bookmark_insert_before (KzBookmark *bookmark,
 {
 	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
 
-	g_signal_emit(G_OBJECT(bookmark),
+	g_signal_emit(bookmark,
 		      kz_bookmark_signals[INSERT_CHILD_SIGNAL], 0,
 		      child, sibling);
 }
@@ -1057,7 +845,7 @@ kz_bookmark_real_remove_child (KzBookmark *bookmark, KzBookmark *child)
 void
 kz_bookmark_remove (KzBookmark *bookmark, KzBookmark *child)
 {
-	g_signal_emit(G_OBJECT(bookmark),
+	g_signal_emit(bookmark,
 		      kz_bookmark_signals[REMOVE_CHILD_SIGNAL], 0,
 		      child);
 }
@@ -1104,7 +892,7 @@ kz_bookmark_get_parent_file (KzBookmark *bookmark)
 	do
 	{
 		parent = kz_bookmark_get_parent(parent);
-		if (KZ_IS_BOOKMARK(parent) && kz_bookmark_is_file(parent))
+		if (KZ_IS_BOOKMARK(parent) && KZ_IS_BOOKMARK_FILE(parent))
 			return parent;
 	} while (parent);
 
@@ -1183,6 +971,59 @@ kz_bookmark_prev (KzBookmark  *bookmark)
 	return NULL;
 }
 
+/* return current activate tab position */
+guint
+kz_bookmark_get_current (KzBookmark *bookmark)
+{
+	if (!kz_bookmark_is_folder(bookmark)) return 0;
+
+	return GPOINTER_TO_UINT(g_object_get_qdata(G_OBJECT(bookmark),
+						   current_pos_quark));
+}
+void
+kz_bookmark_set_current (KzBookmark  *bookmark,
+			 guint pos)
+{
+	if (!kz_bookmark_is_folder(bookmark)) return;
+
+	g_object_set(G_OBJECT(bookmark), "current", pos, NULL);
+}
+
+gboolean
+kz_bookmark_get_lock (KzBookmark  *bookmark)
+{
+	if (!kz_bookmark_is_folder(bookmark)) return FALSE;
+
+	return GPOINTER_TO_INT(g_object_get_qdata(G_OBJECT(bookmark),
+						  lock_quark));
+}
+
+void
+kz_bookmark_set_lock (KzBookmark  *bookmark,
+		      gboolean lock)
+{
+	if (!kz_bookmark_is_folder(bookmark)) return;
+	
+	g_object_set(G_OBJECT(bookmark), "lock", lock, NULL);
+}
+
+gboolean
+kz_bookmark_get_auto_refresh (KzBookmark  *bookmark)
+{
+	if (!kz_bookmark_is_folder(bookmark)) return FALSE;
+
+	return GPOINTER_TO_INT(g_object_get_qdata(G_OBJECT(bookmark),
+						  auto_refresh_quark));
+}
+
+void
+kz_bookmark_set_auto_refresh (KzBookmark  *bookmark,
+			      gboolean auto_refresh)
+{
+	if (!kz_bookmark_is_folder(bookmark)) return;
+
+	g_object_set(G_OBJECT(bookmark), "auto-refresh", auto_refresh, NULL);
+}
 
 static gint 
 compare_func (gconstpointer a, gconstpointer b)
@@ -1213,7 +1054,7 @@ kz_bookmark_sort (KzBookmark *bookmark, const gchar *type)
 
 	g_object_set_qdata(G_OBJECT(bookmark), children_quark, children);
 
-	g_signal_emit(G_OBJECT(bookmark),
+	g_signal_emit(bookmark,
 		      kz_bookmark_signals[CHILDREN_REORDERED_SIGNAL], 0);
 }
 
@@ -1225,283 +1066,39 @@ kz_bookmark_regist_sort_func (const gchar *type, GCompareFunc *func)
 }
 
 
-
-/*****************************************************************************
- *                                                                           *
- *                        load/save booomark file                            *
- *                                                                           *
- *****************************************************************************/
-gboolean
-kz_bookmark_load_start (KzBookmark *bookmark)
+KzBookmark *
+kz_bookmark_find_bookmark_from_uri (KzBookmark *folder, const gchar *key_uri)
 {
-	KzBookmarkState state;
-	KzDownloader *dl;
-	const gchar *uri;
+	GList *children, *node;
+	KzBookmark *bookmark = NULL;
 
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
+	children = kz_bookmark_get_children(folder);
 
-	uri = kz_bookmark_get_location(KZ_BOOKMARK(bookmark));
-	if (!uri) return FALSE;
-
-	state = kz_bookmark_get_state(bookmark);
-	if (state == KZ_BOOKMARK_STATE_LOADING)
-		return TRUE;
-	kz_bookmark_set_state(bookmark, KZ_BOOKMARK_STATE_LOADING);
-
-	g_signal_emit(G_OBJECT(bookmark),
-		      kz_bookmark_signals[LOAD_START_SIGNAL],
-		      0);
-
-	kz_bookmark_remove_all(KZ_BOOKMARK(bookmark));
-
-	dl = kz_downloader_new(uri);
-	g_signal_connect(G_OBJECT(dl), "completed",
-			 G_CALLBACK(cb_downloader_load_complete),
-			 bookmark);
-	g_signal_connect(G_OBJECT(dl), "error",
-			 G_CALLBACK(cb_downloader_load_error),
-			 bookmark);
-	kz_downloader_to_string(dl);
-
-	return TRUE;
-}
-
-
-gboolean
-kz_bookmark_load_in_memory_start (KzBookmark *bookmark)
-{
-	KzBookmarkState state;
-	KzDownloader *dl;
-	const gchar *uri;
-
-	/* FIXME! */
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
-
-	uri = kz_bookmark_get_location(KZ_BOOKMARK(bookmark));
-	if (!uri) return FALSE;
-
-	state = kz_bookmark_get_state(bookmark);
-	if (state == KZ_BOOKMARK_STATE_LOADING)
-		return TRUE;
-	kz_bookmark_set_state(bookmark, KZ_BOOKMARK_STATE_LOADING);
-
-	g_signal_emit(G_OBJECT(bookmark),
-		      kz_bookmark_signals[LOAD_START_SIGNAL],
-		      0);
-
-	kz_bookmark_remove_all(KZ_BOOKMARK(bookmark));
-
-	dl = kz_downloader_new_in_memory(uri);
-	g_signal_connect(G_OBJECT(dl), "completed",
-			 G_CALLBACK(cb_downloader_load_complete),
-			 bookmark);
-	g_signal_connect(G_OBJECT(dl), "error",
-			 G_CALLBACK(cb_downloader_load_error),
-			 bookmark);
-	kz_downloader_to_string(dl);
-
-	return TRUE;
-}
-
-
-void
-kz_bookmark_load_stop (KzBookmark *bookmark)
-{
-	g_warning("kz_bookmark_load_stop() is not implemented yet.");
-}
-
-
-gboolean
-kz_bookmark_save_start (KzBookmark *bookmark)
-{
-	g_warning("kz_bookmark_save_start() is not implemented yet.");
-	return FALSE;
-}
-
-
-void
-kz_bookmark_save_stop (KzBookmark *bookmark)
-{
-	g_warning("kz_bookmark_save_stop() is not implemented yet.");
-}
-
-
-void
-kz_bookmark_load (KzBookmark *bookmark)
-{
-	g_warning("kz_bookmark_load() is not implemented yet.");
-}
-
-
-#warning FIXME! we should use KzIO for writing contents.
-void
-kz_bookmark_save (KzBookmark *bookmark)
-{
-	KzBookmarkFileType *type;
-	KzIO *io;
-	const gchar *file;
-	gchar *str;
-
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-
-	type = kz_bookmark_detect_file_type(bookmark, NULL);
-	if (!type || !type->to_string) return;
-
-	file = kz_bookmark_get_location(bookmark);
-	str = type->to_string(bookmark);
-
-	if (file && *file && str && *str)
+	/* FIXME! Use hash? */
+	for (node = children; node; node = g_list_next(node))
 	{
-		io = kz_io_new(file, KZ_IO_WRITE);
-		kz_io_set_string(io, str);
-		kz_io_start(io);
-		g_object_unref(io);
-	}
-}
-
-
-static gboolean
-idle_load_complete(gpointer data)
-{
-	KzDownloader *dl = data;
-
-	g_object_unref(G_OBJECT(dl));
-
-	return FALSE;
-}
-
-
-static void
-kz_bookmark_load_completed(KzBookmark *bookmark, KzDownloader *dl,
-			   gboolean error)
-{
-	KzBookmarkFileType *type;
-	gchar *buf = NULL;
-	guint size;
-
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-	g_return_if_fail(KZ_IS_DOWNLOADER(dl));
-
-	if (error) goto ERROR;
-
-	buf  = kz_downloader_get_string(dl);
-	size = kz_downloader_get_current_progress(dl);
-
-	g_return_if_fail(buf != NULL);
-	if (size == 0) goto ERROR;
-
-	type = kz_bookmark_detect_file_type(bookmark, buf);
-	if (!type)
-		/* use default? */;
-	if (type && type->from_string)
-	{
-		if (!kz_bookmark_get_file_type(bookmark))
-			kz_bookmark_set_file_type(bookmark, type->file_type);
-		type->from_string(bookmark, buf, size, NULL);
-	}
-
-ERROR:
-	g_free(buf);
-
-	g_signal_handlers_disconnect_by_func
-		(G_OBJECT(dl),
-		 G_CALLBACK(cb_downloader_load_complete),
-		 bookmark);
-	g_signal_handlers_disconnect_by_func
-		(G_OBJECT(dl),
-		 G_CALLBACK(cb_downloader_load_error),
-		 bookmark);
-	kz_bookmark_set_state(bookmark, KZ_BOOKMARK_STATE_NORMAL);
-	if (error)
-	{
-		g_signal_emit(G_OBJECT (bookmark),
-			      kz_bookmark_signals[ERROR_SIGNAL],
-			      0,
-			      "Load failed");
-	}
-	else 
-	{
-		g_signal_emit(G_OBJECT (bookmark),
-			      kz_bookmark_signals[LOAD_COMPLETED_SIGNAL],
-			      0);
-	}
-	g_idle_add(idle_load_complete, dl);
-}
-
-
-static KzBookmarkFileType *
-kz_bookmark_detect_file_type(KzBookmark *bookmark, gchar *buf)
-{
-	GList *node;
-	const gchar *type_str;
-
-	type_str = kz_bookmark_get_file_type(bookmark);
-	g_return_val_if_fail(type_str || buf, NULL);
-
-	for (node = file_types; node; node = g_list_next(node))
-	{
-		KzBookmarkFileType *type = node->data;
- 
-		if (type_str)
+		KzBookmark *child = node->data;
+		const gchar *uri = kz_bookmark_get_link(child);
+		if (uri && key_uri && !g_strcmp(uri, key_uri))
 		{
-			/* Fixed, or already detected */
-
-			if (type->file_type &&
-			    !strcmp(type_str, type->file_type))
-			{
-				return type;
-			}
+			bookmark = child;	
+			break;
 		}
-		else
-		{
-			/* Auto detect */
 
-			if (type && type->is_supported &&
-			    type->is_supported(bookmark, buf))
+		if (kz_bookmark_is_folder(child))
+		{
+			KzBookmark *find;
+			find = kz_bookmark_find_bookmark_from_uri(child,
+								  key_uri);
+			if (find)
 			{
-				return type;
+				bookmark = find;
+				break;
 			}
 		}
 	}
+	g_list_free(children);
 
-	return NULL;
+	return bookmark;
 }
 
-
-static void
-cb_downloader_load_complete(KzDownloader *dl, KzBookmark *bookmark)
-{	
-	kz_bookmark_load_completed(bookmark, dl, FALSE);
-}
-
-
-static void
-cb_downloader_load_error(KzDownloader *dl, KzBookmark *bookmark)
-{	
-	kz_bookmark_load_completed(bookmark, dl, TRUE);
-}
-
-
-static void
-kz_bookmark_set_state (KzBookmark *bookmark, KzBookmarkState state)
-{
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-
-	if (state == KZ_BOOKMARK_STATE_LOADING)
-		g_object_set_qdata(G_OBJECT(bookmark), state_quark,
-				   GINT_TO_POINTER(state));
-	else
-		g_object_set_qdata(G_OBJECT(bookmark), state_quark,
-				   GINT_TO_POINTER(KZ_BOOKMARK_STATE_NORMAL));
-}
-
-
-KzBookmarkState
-kz_bookmark_get_state (KzBookmark *bookmark)
-{
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark),
-			     KZ_BOOKMARK_STATE_NORMAL);
-
-	return GPOINTER_TO_INT(g_object_get_qdata(G_OBJECT(bookmark),
-						  state_quark));
-}
