@@ -17,24 +17,26 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xbel.c,v 1.37 2004/03/20 14:06:01 makeinu Exp $
+ *  $Id: kz-xbel.c,v 1.1.1.1 2005/03/27 12:58:28 poincare Exp $
  */
 
 #include "kazehakase.h"
 #include "kz-xbel.h"
 #include "kz-xml.h"
+#include "utils.h"
+#include "kz-smart-bookmark.h"
 
 #include <string.h>
 #include <stdlib.h>
 
-static gboolean kz_xbel_is_supported       (KzBookmark  *bookmark,
-					    const gchar *buf);
-static void     kz_xbel_init               (KzBookmark  *bookmark);
-static gboolean kz_xbel_from_string        (KzBookmark  *bookmark,
-					    const gchar *buffer,
-					    guint        length,
-					    GError     **error);
-static gchar   *kz_xbel_to_string          (KzBookmark  *bookmark);
+static gboolean kz_xbel_is_supported       (KzBookmarkFile  *bookmark,
+					    const gchar     *buf);
+static void     kz_xbel_init               (KzBookmarkFile  *bookmark);
+static gboolean kz_xbel_from_string        (KzBookmarkFile  *bookmark,
+					    const gchar     *buffer,
+					    guint            length,
+					    GError         **error);
+static gchar   *kz_xbel_to_string          (KzBookmarkFile   *bookmark);
 
 static void     kz_xbel_notify             (GObject     *object,
 					    GParamSpec  *pspec,
@@ -94,7 +96,7 @@ kz_xbel_get_file_types (gint idx)
 
 
 static gboolean
-kz_xbel_is_supported(KzBookmark *bookmark, const gchar *buf)
+kz_xbel_is_supported(KzBookmarkFile *bookmark, const gchar *buf)
 {
 	const gchar *pos;
 
@@ -117,7 +119,7 @@ kz_xbel_is_supported(KzBookmark *bookmark, const gchar *buf)
 
 
 static void
-kz_xbel_init (KzBookmark *bookmark)
+kz_xbel_init (KzBookmarkFile *bookmark)
 {
 	KzXML *xml;
 
@@ -132,28 +134,27 @@ kz_xbel_init (KzBookmark *bookmark)
 	g_object_set_qdata_full(G_OBJECT(bookmark), xml_quark,
 				xml, (GDestroyNotify) g_object_unref);
 
-	bookmark->flags |= KZ_BOOKMARK_FILE_FLAG;
 	g_object_set(G_OBJECT(bookmark),
 		     "type", KZ_BOOKMARK_PURE_FOLDER,
 		     NULL);
 
-	g_signal_connect(G_OBJECT(bookmark), "notify",
+	g_signal_connect(bookmark, "notify",
 			 G_CALLBACK(kz_xbel_notify), xml);
-	g_signal_connect_after(G_OBJECT(bookmark), "insert-child",
+	g_signal_connect_after(bookmark, "insert-child",
 			       G_CALLBACK(cb_bookmark_insert_child), xml);
-	g_signal_connect_after(G_OBJECT(bookmark), "remove-child",
+	g_signal_connect_after(bookmark, "remove-child",
 			       G_CALLBACK(cb_bookmark_remove_child), xml);
 }
 
 
 static gboolean
-kz_xbel_from_string (KzBookmark *bookmark,
+kz_xbel_from_string (KzBookmarkFile *bookmark,
 		     const gchar *buffer, guint length,
 		     GError **error)
 {
 	KzXML *xml;
 
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), FALSE);
+	g_return_val_if_fail(KZ_IS_BOOKMARK_FILE(bookmark), FALSE);
 
 	xml = g_object_get_qdata(G_OBJECT(bookmark), xml_quark);
 	g_return_val_if_fail(KZ_IS_XML(xml), FALSE);
@@ -165,13 +166,14 @@ kz_xbel_from_string (KzBookmark *bookmark,
 		const gchar *bookmark_title;
 
 		/*
-		 *  <xbel version=1.0 folded=none>\n (space1)
+		 *  <xbel version=1.0 folded=no>\n (space1)
 		 *  <title>Bookmarks</title>\n       (space2)
 		 *  </xbel>
 		 */
 		node = kz_xml_element_node_new("xbel");
 		kz_xml_node_set_attr(node, "version", "1.0");
-		kz_xml_node_set_attr(node, "folded", "none");	
+		kz_xml_node_set_attr(node, "folded", "no");	
+		kz_xml_node_set_attr(node, "xmlns:kz", KAZEHAKASE_URI"2004");	
 		/* kz_xml_node_set_attr(node, "id", "????????"); */
 		kz_xml_node_append_child(xml->root, node);
 
@@ -183,7 +185,7 @@ kz_xbel_from_string (KzBookmark *bookmark,
 			title_node = kz_xml_element_node_new("title");
 			kz_xml_node_append_child(node, title_node);
 
-			bookmark_title = kz_bookmark_get_title(bookmark);
+			bookmark_title = kz_bookmark_get_title(KZ_BOOKMARK(bookmark));
 			if(bookmark_title)
 				title = kz_xml_text_node_new(bookmark_title);
 			else
@@ -196,18 +198,18 @@ kz_xbel_from_string (KzBookmark *bookmark,
 		kz_xml_node_append_child(node, space);
 	}
 
-	kz_xbel_build_tree(bookmark);
+	kz_xbel_build_tree(KZ_BOOKMARK(bookmark));
 
 	return TRUE;
 }
 
 
 static gchar *
-kz_xbel_to_string (KzBookmark *bookmark)
+kz_xbel_to_string (KzBookmarkFile *bookmark)
 {
 	KzXML *xml;
 
-	g_return_val_if_fail(KZ_IS_BOOKMARK(bookmark), NULL);
+	g_return_val_if_fail(KZ_IS_BOOKMARK_FILE(bookmark), NULL);
 
 	xml = g_object_get_qdata(G_OBJECT(bookmark), xml_quark);
 	g_return_val_if_fail(KZ_IS_XML(xml), NULL);
@@ -253,88 +255,7 @@ kz_xbel_notify (GObject *object, GParamSpec *pspec, KzXML *xml)
 		xml_node_set_title(node, title);
 		g_free(title);
 	}
-#if 0
-	else if (!strcmp(prop, "link"))
-	{
-		gchar *uri = g_value_dup_string(&value);
-
-		kz_xml_node_set_attr(node, "href", uri);
-		g_free(uri);
-	}
-	else if (!strcmp(prop, "description"))
-	{
-		gchar *desc = g_value_dup_string(&value);
-
-		xml_node_set_description(node, desc);
-		g_free(desc);
-	}
-#endif
-}
-
-
-static void
-parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
-{
-	KzXMLNode *node;
-
-	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
-
-	if (!kz_xml_node_is_element(parent)) return;
-
-	for (node = kz_xml_node_first_child(parent);
-	     node;
-	     node = kz_xml_node_next(node))
-	{
-		if (!kz_xml_node_is_element(node)) continue;
-
-		if (kz_xml_node_name_is(node, "metadata"))
-		{
-			const gchar *owner;
-
-			/* We should ignore other owners such as Galeon. */
-			owner = kz_xml_node_get_attr(node, "owner");
-			if (!owner || strcmp(owner, KAZEHAKASE_URI))
-				continue;
-
-			parse_metadata_node(bookmark, node);
-			/*
-			if (kz_bookmark_get_owner(bookmark))
-			{
-				g_warning("metadata owner attribute is duplicated!");
-				continue;
-			}
-			owner = kz_xml_node_get_attr(node, "owner");
-			kz_bookmark_set_owner(folder, owner);
-			*/
-		}
-		else if (kz_xml_node_name_is(node, "location"))
-		{
-			/*
-			 * "location" element is already parsed at
-			 * parse_child_node().
-			 */
-			if (!kz_bookmark_is_file(bookmark))
-			{
-				g_warning("kzXBEL: The bookmark has location, "
-					  "but it's not a folder element!");
-			}
-		}
-		else if (kz_xml_node_name_is(node, "update_interval"))
-		{
-			gchar *interval_str;
-			guint interval;
-
-			if (kz_bookmark_get_interval(bookmark))
-			{
-				g_warning("update_interval element is duplicated!");
-				continue;
-			}
-			interval_str = kz_xml_node_to_str(node);
-			interval = atoi(interval_str);
-			kz_bookmark_set_interval(bookmark, interval);
-			g_free(interval_str);
-		}
-	}	
+	g_value_unset(&value);
 }
 
 
@@ -358,6 +279,108 @@ xml_node_get_named_node (KzXMLNode *parent, const gchar *name)
 }
 
 
+static void
+xml_node_set_location (KzXMLNode *node, KzBookmark *bookmark);
+static void
+xml_node_set_interval (KzXMLNode *node, KzBookmark *bookmark);
+
+static void
+parse_metadata_node (KzBookmark *bookmark, KzXMLNode *parent)
+{
+	KzXMLNode *node;
+
+	g_return_if_fail(KZ_IS_BOOKMARK(bookmark));
+
+	if (!kz_xml_node_is_element(parent)) return;
+
+	for (node = kz_xml_node_first_child(parent);
+	     node;
+	     node = kz_xml_node_next(node))
+	{
+		const GList *attrs, *a_node;
+		const gchar *owner;
+		const gchar *smart_regex  = NULL;
+		const gchar *smart_uri    = NULL;
+		const gchar *smart_encode = NULL;
+		gboolean urlencode = FALSE;
+		gint interval;
+
+		if (!kz_xml_node_is_element(node)) continue;
+
+		if (!kz_xml_node_name_is(node, "metadata")) continue;
+			
+		/* We should ignore other owners such as Galeon. */
+		owner = kz_xml_node_get_attr(node, "owner");
+		if (!owner || strcmp(owner, KAZEHAKASE_URI))
+			continue;
+			
+		attrs = kz_xml_node_get_attrs(node);
+
+		for (a_node = attrs; a_node; a_node = g_list_next(a_node))
+		{
+			KzXMLAttr *attr = a_node->data;
+
+			if (!strncmp(attr->name, "kz:update_interval", 118))
+			{
+				interval = atoi(attr->value);
+				kz_bookmark_file_set_interval(KZ_BOOKMARK_FILE(bookmark), interval);
+			}
+			else if (!strncmp(attr->name, "kz:smart_regex", 14))
+			{
+				smart_regex = attr->value;
+			}
+			else if (!strncmp(attr->name, "kz:smart_uri", 12))
+			{
+				smart_uri = attr->value;
+			}
+			else if (!strncmp(attr->name, "kz:smart_encode", 15))
+			{
+				smart_encode = attr->value;
+			}
+			else if (!strncmp(attr->name, "kz:smart_urlencode", 16))
+			{
+				urlencode = TRUE;
+			}
+			else if (!strncmp(attr->name, "kz:xmlrpc", 9))
+			{
+				kz_bookmark_file_set_xmlrpc(KZ_BOOKMARK_FILE(bookmark), attr->value);
+			}
+			else if (!strncmp(attr->name, "kz:xmlrpc_user", 14))
+			{
+				kz_bookmark_file_set_xmlrpc_user(KZ_BOOKMARK_FILE(bookmark), attr->value);
+			}
+			else if (!strncmp(attr->name, "kz:xmlrpc_pass", 14))
+			{
+				kz_bookmark_file_set_xmlrpc_pass(KZ_BOOKMARK_FILE(bookmark), attr->value);
+			}
+			else if (!strncmp(attr->name, "kz:current_position", 18))
+			{
+				gint pos = atoi(attr->value);
+				kz_bookmark_set_current(bookmark, pos);
+			}
+			else if (!strncmp(attr->name, "kz:lock", 7))
+			{
+				if (!strcmp(attr->value, "yes"))
+					kz_bookmark_set_lock(bookmark, TRUE);
+			}
+			else if (!strncmp(attr->name, "kz:auto_refresh", 15))
+			{
+				if (!strcmp(attr->value, "yes"))
+					kz_bookmark_set_auto_refresh(bookmark, TRUE);
+			}
+
+		}
+
+		if (smart_regex && smart_uri)
+		{
+			 kz_smart_bookmark_append_property(
+				KZ_SMART_BOOKMARK(bookmark),
+				smart_regex, smart_uri, 
+				smart_encode, urlencode);
+		}
+	}	
+}
+
 static KzXMLNode *
 xml_node_find_metadata_node (KzXMLNode *parent)
 {
@@ -370,24 +393,64 @@ xml_node_find_metadata_node (KzXMLNode *parent)
 	     node;
 	     node = kz_xml_node_next(node))
 	{
-		if (kz_xml_node_name_is(node, "metadata"))
-		{
-			const gchar *owner;
+		const gchar *owner;
+	
+		if (!kz_xml_node_name_is(node, "metadata"))
+			continue;
 
-			owner = kz_xml_node_get_attr(node, "owner");
-			if (owner && !strcmp(owner, KAZEHAKASE_URI))
-				return node;
-		}
+		owner = kz_xml_node_get_attr(node, "owner");
+		if (owner && !strcmp(owner, KAZEHAKASE_URI))
+			return node;
 	}
 
 	return NULL;
 }
 
 
+static gboolean
+xml_node_has_smart_property (KzXMLNode *parent)
+{
+	const GList *attrs, *list;
+	KzXMLNode *info_node, *node;
+	gboolean ret = FALSE;
+			
+	info_node = xml_node_get_named_node(parent, "info");
+	if (!info_node) return FALSE;
+	
+	for (node = kz_xml_node_first_child(info_node);
+	     node;
+	     node = kz_xml_node_next(node))
+	{
+		const gchar *owner;
+		if (!kz_xml_node_name_is(node, "metadata"))
+			continue;
+
+		owner = kz_xml_node_get_attr(node, "owner");
+		if (!owner || strcmp(owner, KAZEHAKASE_URI))
+			continue;
+
+		attrs = kz_xml_node_get_attrs(node);
+
+		for (list = attrs; list; list = g_list_next(list))
+		{
+			KzXMLAttr *attr = list->data;
+
+			if (!strncmp(attr->name, "kz:smart", 7))
+			{
+				ret = TRUE;
+				break;
+			}
+		}
+		if (ret) break;
+	}
+	return ret;
+}
+
+
 static gboolean
 xml_node_has_location (KzXMLNode *parent)
 {
-	KzXMLNode *child, *gchild;
+	KzXMLNode *child;
 
 	g_return_val_if_fail(parent, FALSE);
 
@@ -397,17 +460,18 @@ xml_node_has_location (KzXMLNode *parent)
 	child = xml_node_find_metadata_node(parent);
 	if (!child) return FALSE;
 
-	gchild = xml_node_get_named_node(child, "location");
-	if (gchild) return TRUE;
-
+	if (kz_xml_node_get_attr(child, "kz:location"))
+		return TRUE;
+	
 	return FALSE;
 }
 
 
-static gchar *
+static const gchar *
 xml_node_get_location (KzXMLNode *parent)
 {
-	KzXMLNode *child, *gchild;
+	KzXMLNode *child;
+	const gchar *attr;
 
 	g_return_val_if_fail(parent, FALSE);
 
@@ -417,10 +481,11 @@ xml_node_get_location (KzXMLNode *parent)
 	child = xml_node_find_metadata_node(parent);
 	if (!child) return NULL;
 
-	gchild = xml_node_get_named_node(child, "location");
-	if (!gchild) return NULL;
-
-	return kz_xml_node_to_str(gchild);
+	/* Does metadata node have location attr? */
+	attr = kz_xml_node_get_attr(child, "kz:location");
+	if (!attr) return NULL;
+	
+	return attr;
 }
 
 
@@ -468,18 +533,19 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 		}
 		else if (kz_xml_node_name_is(node, "folder"))
 		{
-			KzBookmark *folder = kz_bookmark_pure_folder_new();
+			KzBookmark *folder;
 			const gchar *folded_str;
+			const gchar *id;
 			gboolean folded;
 
 			if (xml_node_has_location(node))
 			{
 #warning FIXME! If the file does not exit, we should creat it.
-				gchar *location = xml_node_get_location(node);
+				const gchar *location = xml_node_get_location(node);
 
-				folder = kz_bookmark_file_new(location,
-							      NULL, NULL);
-				kz_bookmark_load_in_memory_start(folder);
+				folder = KZ_BOOKMARK(kz_bookmark_file_new(location,
+							      NULL, NULL));
+				kz_bookmark_file_load_start(KZ_BOOKMARK_FILE(folder));
 			}
 			else
 			{
@@ -490,6 +556,10 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			folded = folded_str && !strcmp(folded_str, "yes");
 			kz_bookmark_set_folded(folder, folded);
 
+			id = kz_xml_node_get_attr(node, "id");
+			if (id)
+				kz_bookmark_set_id(folder, id);
+
 			BOOKMARK_SET_BUILDING(folder);
 			g_object_set_qdata(G_OBJECT(folder),
 					   node_quark, node);
@@ -497,25 +567,58 @@ parse_child_node (KzBookmark *bookmark, KzXMLNode *parent)
 			parse_child_node(folder, node);
 			BOOKMARK_UNSET_BUILDING(folder);
 
-			if (kz_bookmark_is_file(folder))
-			{
-				kz_bookmark_remove_all(folder);
-				kz_bookmark_load_in_memory_start(folder);
-			}
-
 			g_object_unref(folder);
 		}
 		else if (kz_xml_node_name_is(node, "bookmark"))
 		{
-			KzBookmark *child_bookmark = kz_bookmark_new();
-			const gchar *uri = kz_xml_node_get_attr(node, "href");
+			KzBookmark *child_bookmark;
+			const GList *attrs = kz_xml_node_get_attrs(node);
+			const GList *list;
+
+			if (xml_node_has_smart_property(node))
+				child_bookmark = kz_smart_bookmark_new();
+			else
+				child_bookmark = kz_bookmark_new();
 
 			BOOKMARK_SET_BUILDING(bookmark);
 			g_object_set_qdata(G_OBJECT(child_bookmark),
 					   node_quark, node);
-			kz_bookmark_set_link(child_bookmark, uri);
-			kz_bookmark_append(bookmark, child_bookmark);
+			for (list = attrs; list; list = g_list_next(list))
+			{
+				KzXMLAttr *attr = list->data;
+				const gchar *name = attr->name;
+
+				if (!strcmp(name, "href"))
+				{
+					kz_bookmark_set_link(child_bookmark, attr->value);
+				}
+				else if (!strcmp(name, "added"))
+				{
+					guint t = 0;
+					if (str_isdigit(attr->value))
+						t = atoi(attr->value);	
+					kz_bookmark_set_added_time(child_bookmark, t);
+				}
+				else if (!strcmp(name, "visited"))
+				{
+					guint t = 0;
+					if (str_isdigit(attr->value))
+						t = atoi(attr->value);	
+					kz_bookmark_set_last_visited(child_bookmark, t);
+				}
+				else if (!strcmp(name, "id"))
+				{
+					kz_bookmark_set_id(child_bookmark, attr->value);
+				}
+			}
+
 			parse_child_node(child_bookmark, node);
+			/*
+			 * To append child is after parsing child's nodes,
+			 * because smart bookmark action needs bookmark title 
+			 * in child's nodes.
+			 */
+			kz_bookmark_append(bookmark, child_bookmark);
 			BOOKMARK_UNSET_BUILDING(bookmark);
 			g_object_unref(child_bookmark);
 		}
@@ -624,10 +727,6 @@ xml_node_append_metadata_node (KzXMLNode *node)
 	kz_xml_node_set_attr(metadata_node, "owner", KAZEHAKASE_URI);
 	kz_xml_node_append_child(info_node, metadata_node);
 
-	/* space2 */
-	space = kz_xml_text_node_new("\n");
-	kz_xml_node_append_child(metadata_node, space);
-
 	/* space3 */
 	space = kz_xml_text_node_new("\n");
 	kz_xml_node_append_child(info_node, space);
@@ -640,40 +739,17 @@ static void
 xml_node_set_location (KzXMLNode *node, KzBookmark *bookmark)
 {
 	const gchar *location;
-	KzXMLNode *metadata_node, *location_node,
-		  *text_node, *space;
+	KzXMLNode *metadata_node;
 
-	location = kz_bookmark_get_location(bookmark);
+	location = kz_bookmark_file_get_location(KZ_BOOKMARK_FILE(bookmark));
 	g_return_if_fail(location);
 
-	metadata_node = xml_node_append_metadata_node(node);
-	location_node = xml_node_get_named_node(metadata_node, "location");
+	metadata_node = xml_node_find_metadata_node(node);
+	
+	if (!metadata_node)
+		metadata_node = xml_node_append_metadata_node(node);
 
-	if (location_node)
-	{
-		KzXMLNode *child;
-
-		/* remove all child node */
-		child = kz_xml_node_first_child(location_node);
-		while (child)
-		{
-			KzXMLNode *next = kz_xml_node_next(child);
-			kz_xml_node_remove_child(location_node, child);
-			kz_xml_node_unref(child);
-			child = next;
-		}
-	}
-	else
-	{
-		location_node = kz_xml_element_node_new("location");
-		kz_xml_node_append_child(metadata_node, location_node);
-	}
-
-	text_node = kz_xml_text_node_new(location);
-	kz_xml_node_append_child(location_node, text_node);
-
-	space = kz_xml_text_node_new("\n");
-	kz_xml_node_append_child(metadata_node, space);
+	kz_xml_node_set_attr(metadata_node, "kz:location", location);
 }
 
 
@@ -681,46 +757,125 @@ static void
 xml_node_set_interval (KzXMLNode *node, KzBookmark *bookmark)
 {
 	gchar *interval;
-	KzXMLNode *metadata_node, *interval_node,
-		  *text_node, *space;
+	KzXMLNode *metadata_node;
 
-	interval = g_strdup_printf("%d", kz_bookmark_get_interval(bookmark));
+	interval = g_strdup_printf("%d", kz_bookmark_file_get_interval(KZ_BOOKMARK_FILE(bookmark)));
 
-	metadata_node = xml_node_append_metadata_node(node);
-	interval_node = xml_node_get_named_node(metadata_node,
-						"update_interval");
+	metadata_node = xml_node_find_metadata_node(node);
+	
+	if (!metadata_node)
+		metadata_node = xml_node_append_metadata_node(node);
 
 	/* update_interval */
-	if (interval_node)
-	{
-		KzXMLNode *child;
-
-		/* remove all child node */
-		child = kz_xml_node_first_child(interval_node);
-		while (child)
-		{
-			KzXMLNode *next = kz_xml_node_next(child);
-			kz_xml_node_remove_child(interval_node, child);
-			kz_xml_node_unref(child);
-			child = next;
-		}
-	}
-	else
-	{
-		interval_node = kz_xml_element_node_new("update_interval");
-		kz_xml_node_append_child(metadata_node, interval_node);
-	}
-
-	text_node = kz_xml_text_node_new(interval);
-	kz_xml_node_append_child(interval_node, text_node);
-
-	space = kz_xml_text_node_new("\n");
-	kz_xml_node_append_child(metadata_node, space);
+	kz_xml_node_set_attr(metadata_node, "kz:update_interval", interval);
 
 	g_free(interval);
 }
 
 
+static void
+xml_node_set_smart_list (KzXMLNode *node,
+			 KzBookmark *bookmark,
+		       	 const GList *smart_list)
+{
+	const GList *smart_node;	
+	KzXMLNode *info_node, *child_node;
+
+	/* get the info node, create it if not exist */
+	info_node = xml_node_get_named_node(node, "info");
+	if (!info_node)
+	{
+		KzXMLNode *space;
+
+		info_node = kz_xml_element_node_new("info");
+		kz_xml_node_append_child(node, info_node);
+
+		/* space1 */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(info_node, space);
+
+		/* space4 */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+	}
+
+	/* remove all previous smart properties */
+	child_node = kz_xml_node_first_child(info_node);
+	while (child_node)
+	{
+		if (kz_xml_node_name_is(child_node, "metadata"))
+		{
+			const gchar *owner, *smart_regex;
+			owner = kz_xml_node_get_attr(child_node, "owner");
+			smart_regex = kz_xml_node_get_attr(child_node,
+							   "kz:smart_regex");
+			if (owner && 
+			   !strcmp(owner, KAZEHAKASE_URI) &&
+			    smart_regex)
+			{
+				KzXMLNode *tmp = child_node;
+				child_node = kz_xml_node_next(child_node);
+				kz_xml_node_remove_child(info_node,
+							 tmp);
+				kz_xml_node_unref(tmp);
+
+				/* space is also removed */
+				if (kz_xml_node_is_space(child_node))
+				{
+					KzXMLNode *tmp = child_node;
+					child_node = kz_xml_node_next(child_node);
+					kz_xml_node_remove_child(info_node,
+								 tmp);
+					kz_xml_node_unref(tmp);
+				}
+			}
+			else
+				child_node = kz_xml_node_next(child_node);
+		}
+		else
+			child_node = kz_xml_node_next(child_node);
+	}
+
+	/* append new smart properties */
+	for (smart_node = smart_list; smart_node; smart_node = g_list_next(smart_node))
+	{
+		KzSmartBookmarkProperty *prop;
+		KzXMLNode *metadata_node, *space;
+		
+		metadata_node = kz_xml_element_node_new("metadata");
+		kz_xml_node_set_attr(metadata_node, "owner", KAZEHAKASE_URI);
+		kz_xml_node_append_child(info_node, metadata_node);
+
+		/* space */
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(info_node, space);
+
+		prop = smart_node->data;
+
+		if (prop->regex)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_regex", prop->regex);
+		}
+		if (prop->uri)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_uri", prop->uri);
+		}
+		if (prop->encode)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_encode", prop->encode);
+		}
+		if (prop->urlencode)
+		{
+			kz_xml_node_set_attr(metadata_node,
+					     "kz:smart_urlencode", "yes");
+		}
+	}
+}
+
+
 static KzXMLNode *
 create_xml_node (KzBookmark *bookmark)
 {
@@ -732,10 +887,10 @@ create_xml_node (KzBookmark *bookmark)
 	{
 		node = kz_xml_element_node_new("separator");
 	}
-	else if (kz_bookmark_is_file(bookmark))
+	else if (KZ_IS_BOOKMARK_FILE(bookmark))
 	{
-		const gchar *location = kz_bookmark_get_location(bookmark);
-		guint interval        = kz_bookmark_get_interval(bookmark);
+		const gchar *location = kz_bookmark_file_get_location(KZ_BOOKMARK_FILE(bookmark));
+		guint interval        = kz_bookmark_file_get_interval(KZ_BOOKMARK_FILE(bookmark));
 
 		if (!location || !*location)
 		{
@@ -762,6 +917,10 @@ create_xml_node (KzBookmark *bookmark)
 	}
 	else if (kz_bookmark_is_folder(bookmark))
 	{
+		KzXMLNode *metanode;
+		gint pos = kz_bookmark_get_current(bookmark);
+		gboolean lock = kz_bookmark_get_lock(bookmark);
+		gboolean auto_refresh = kz_bookmark_get_auto_refresh(bookmark);
 		/*
 		 * <folder>\n
 		 * <title/>
@@ -775,11 +934,68 @@ create_xml_node (KzBookmark *bookmark)
 		kz_xml_node_append_child(node, space);
 
 		xml_node_append_title (node, bookmark);
+		
+		if (pos || lock || auto_refresh)
+		{
+			metanode = xml_node_find_metadata_node(node);
+	
+			if (!metanode)
+				metanode = xml_node_append_metadata_node(node);
+
+			if (pos)
+			{
+				gchar *pos_str = g_strdup_printf("%d", pos);
+				kz_xml_node_set_attr(metanode, "kz:current_position", pos_str);
+			}
+			if (lock)
+			{
+				const gchar *str;
+				if (lock)
+					str = "yes";
+				else
+					str = "no";
+				kz_xml_node_set_attr(metanode, "kz:lock", str);
+			}
+			if (auto_refresh)
+			{
+				const gchar *str;
+				if (auto_refresh)
+					str = "yes";
+				else
+					str = "no";
+				kz_xml_node_set_attr(metanode, "kz:auto_refresh", str);
+			}
+
+		}
+	}
+	else if (KZ_IS_SMART_BOOKMARK(bookmark))
+	{
+		const GList *smart_list;
+		const gchar *uri = kz_bookmark_get_link(bookmark);
+		
+		smart_list = kz_smart_bookmark_get_smart_list(
+				KZ_SMART_BOOKMARK(bookmark));
+	
+		node = kz_xml_element_node_new("bookmark");
+		if (uri)
+			kz_xml_node_set_attr(node, "href", uri);
+		if (smart_list)
+		{
+			xml_node_set_smart_list(node, 
+						bookmark,
+						smart_list);
+		}
+		space = kz_xml_text_node_new("\n");
+		kz_xml_node_append_child(node, space);
+
+		xml_node_append_title (node, bookmark);
 	}
 	else
 	{
 		const gchar *uri = kz_bookmark_get_link(bookmark);
-
+		const gchar *id = kz_bookmark_get_id(bookmark);
+		guint visited = kz_bookmark_get_last_visited(bookmark);
+		guint added = kz_bookmark_get_added_time(bookmark);
 		/*
 		 * <bookmark>\n
 		 * <title/>
@@ -789,6 +1005,20 @@ create_xml_node (KzBookmark *bookmark)
 		node = kz_xml_element_node_new("bookmark");
 		if (uri)
 			kz_xml_node_set_attr(node, "href", uri);
+		if (id)
+			kz_xml_node_set_attr(node, "id", id);
+		if (visited != 0)
+		{
+			gchar *str = g_strdup_printf("%d", visited);
+			kz_xml_node_set_attr(node, "visited", str);
+			g_free(str);
+		}
+		if (added != 0)
+		{
+			gchar *str = g_strdup_printf("%d", added);
+			kz_xml_node_set_attr(node, "added", str);
+			g_free(str);
+		}
 
 		space = kz_xml_text_node_new("\n");
 		kz_xml_node_append_child(node, space);
@@ -803,17 +1033,17 @@ create_xml_node (KzBookmark *bookmark)
 static void
 kz_xbel_connect_signals (KzBookmark *bookmark)
 {
-	g_signal_connect(G_OBJECT(bookmark), "notify",
+	g_signal_connect(bookmark, "notify",
 			 G_CALLBACK(cb_bookmark_notify), NULL);
 
-	if (kz_bookmark_is_file(bookmark)) return;
+	if (KZ_IS_BOOKMARK_FILE(bookmark)) return;
 	if (!kz_bookmark_is_folder(bookmark)) return;
 
 	{
-		g_signal_connect_after(G_OBJECT(bookmark), "insert-child",
+		g_signal_connect_after(bookmark, "insert-child",
 				       G_CALLBACK(cb_bookmark_insert_child),
 				       NULL);
-		g_signal_connect_after(G_OBJECT(bookmark), "remove-child",
+		g_signal_connect_after(bookmark, "remove-child",
 				       G_CALLBACK(cb_bookmark_remove_child),
 				       NULL);
 	}
@@ -837,18 +1067,18 @@ static void
 kz_xbel_disconnect_signals (KzBookmark *bookmark)
 {
 	g_signal_handlers_disconnect_by_func
-		(G_OBJECT(bookmark),
+		(bookmark,
 		 G_CALLBACK(cb_bookmark_notify), NULL);
 
-	if (kz_bookmark_is_file(bookmark)) return;
+	if (KZ_IS_BOOKMARK_FILE(bookmark)) return;
 	if (!kz_bookmark_is_folder(bookmark)) return;
 
 	{
 		g_signal_handlers_disconnect_by_func
-			(G_OBJECT(bookmark),
+			(bookmark,
 			 G_CALLBACK(cb_bookmark_insert_child), NULL);
 		g_signal_handlers_disconnect_by_func
-			(G_OBJECT(bookmark),
+			(bookmark,
 			 G_CALLBACK(cb_bookmark_remove_child), NULL);
 	}
 
@@ -971,59 +1201,6 @@ xml_node_set_description (KzXMLNode *parent, const gchar *desc)
 }
 
 
-#if 0
-static void
-xml_node_set_element (KzXMLNode *parent,
-		      const gchar *contents, const gchar *name)
-{
-	KzXMLNode *element_node = NULL, *node, *child;
-
-	g_return_if_fail(parent);
-	g_return_if_fail(name);
-
-	node = kz_xml_node_first_child(parent);
-
-	/* find node */
-	for (; node; node = kz_xml_node_next(node))
-	{
-		if (kz_xml_node_name_is(node, name))
-		{
-			element_node = node;
-			break;
-		}
-	}
-
-	/* remove old element node, or create new element node */
-	if (element_node)
-	{
-		child = kz_xml_node_first_child(node);
-		while(child)
-		{
-			KzXMLNode *next = kz_xml_node_next(child);
-
-			child = kz_xml_node_remove_child(node, child);
-			kz_xml_node_unref(child);
-			child = next;
-		}
-	}
-	else
-	{
-		KzXMLNode *space;
-
-		element_node = kz_xml_element_node_new(name);
-		kz_xml_node_append_child(parent, element_node);
-
-		space = kz_xml_text_node_new("\n");
-		kz_xml_node_append_child(parent, space);
-	}
-
-	/* set new element */
-	child = kz_xml_text_node_new(contents);
-	kz_xml_node_append_child(element_node, child);
-}
-#endif
-
-
 static void
 cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 {
@@ -1051,7 +1228,14 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 	/*
 	 * FIXME! We should use hash table.
 	 */
-	if (!strcmp(prop, "title"))
+	if (!strcmp(prop, "id"))
+	{
+		gchar *id = g_value_dup_string(&value);
+
+		kz_xml_node_set_attr(node, "id", id);
+		g_free(id);
+	}
+	else if (!strcmp(prop, "title"))
 	{
 		gchar *title = g_value_dup_string(&value);
 
@@ -1075,13 +1259,117 @@ cb_bookmark_notify (GObject *object, GParamSpec *pspec)
 	else if (!strcmp(prop, "location"))
 	{
 		xml_node_set_location(node, bookmark);
-		/* xml_node_set_metadata(node, "location", str) */
 	}
 	else if (!strcmp(prop, "interval"))
 	{
 		xml_node_set_interval(node, bookmark);
-		/* xml_node_set_metadata(node, "update_interval", str); */
 	}
+	else if (!strcmp(prop, "last-visited"))
+	{
+		guint t = g_value_get_uint(&value);
+		gchar *t_str = g_strdup_printf("%d", t);
+		kz_xml_node_set_attr(node, "visited", t_str);
+		g_free(t_str);
+	}
+	else if (!strcmp(prop, "added-time"))
+	{
+		guint t = g_value_get_uint(&value);
+		gchar *t_str = g_strdup_printf("%d", t);
+		kz_xml_node_set_attr(node, "added", t_str);
+		g_free(t_str);
+	}
+	else if (!strcmp(prop, "smart-list"))
+	{
+		const GList *smart_list;
+		smart_list =  g_value_get_pointer(&value);
+		xml_node_set_smart_list(node, bookmark, smart_list);
+	}
+	else if (!strcmp(prop, "xmlrpc"))
+	{
+		KzXMLNode *metanode;
+		gchar *xmlrpc = g_value_dup_string(&value);
+		metanode = xml_node_find_metadata_node(node);
+	
+		if (!metanode)
+			metanode = xml_node_append_metadata_node(node);
+
+		kz_xml_node_set_attr(metanode, "kz:xmlrpc", xmlrpc);
+		g_free(xmlrpc);
+	}
+	else if (!strcmp(prop, "xmlrpc-user"))
+	{
+		KzXMLNode *metanode;
+		gchar *xmlrpc = g_value_dup_string(&value);
+		metanode = xml_node_find_metadata_node(node);
+	
+		if (!metanode)
+			metanode = xml_node_append_metadata_node(node);
+
+		kz_xml_node_set_attr(metanode, "kz:xmlrpc_user", xmlrpc);
+		g_free(xmlrpc);
+	}
+	else if (!strcmp(prop, "xmlrpc-pass"))
+	{
+		KzXMLNode *metanode;
+		gchar *xmlrpc = g_value_dup_string(&value);
+		metanode = xml_node_find_metadata_node(node);
+	
+		if (!metanode)
+			metanode = xml_node_append_metadata_node(node);
+
+		kz_xml_node_set_attr(metanode, "kz:xmlrpc_user", xmlrpc);
+		g_free(xmlrpc);
+	}
+	else if (!strcmp(prop, "current"))
+	{
+		KzXMLNode *metanode;
+		guint pos = g_value_get_uint(&value);
+		gchar *pos_str = g_strdup_printf("%d", pos);
+		metanode = xml_node_find_metadata_node(node);
+	
+		if (!metanode)
+			metanode = xml_node_append_metadata_node(node);
+
+		kz_xml_node_set_attr(metanode, "kz:current_position", pos_str);
+		g_free(pos_str);
+	}
+	else if (!strcmp(prop, "lock"))
+	{
+		KzXMLNode *metanode;
+		const gchar *str;
+		gboolean lock = g_value_get_boolean(&value);
+		
+		if (lock)
+			str = "yes";
+		else
+			str = "no";
+
+		metanode = xml_node_find_metadata_node(node);
+	
+		if (!metanode)
+			metanode = xml_node_append_metadata_node(node);
+
+		kz_xml_node_set_attr(metanode, "kz:lock", str);
+	}
+	else if (!strcmp(prop, "auto-refresh"))
+	{
+		KzXMLNode *metanode;
+		const gchar *str;
+		gboolean auto_refresh = g_value_get_boolean(&value);
+		
+		if (auto_refresh)
+			str = "yes";
+		else
+			str = "no";
+
+		metanode = xml_node_find_metadata_node(node);
+	
+		if (!metanode)
+			metanode = xml_node_append_metadata_node(node);
+
+		kz_xml_node_set_attr(metanode, "kz:auto_refresh", str);
+	}
+	g_value_unset(&value);
 }
 
 
@@ -1143,7 +1431,7 @@ kz_xbel_insert_xml_node (KzBookmark *bookmark,
 	space = kz_xml_text_node_new("\n");
 	kz_xml_node_insert_before(parent_node, space, kz_xml_node_next(node));
 
-	if (kz_bookmark_is_folder(bookmark) && !kz_bookmark_is_file(bookmark))
+	if (kz_bookmark_is_folder(bookmark) && !KZ_IS_BOOKMARK_FILE(bookmark))
 	{
 		GList *node, *children;
 
