@@ -22,9 +22,9 @@
 
 #include <string.h>
 #include <gdk/gdkkeysyms.h>
+#include <glib/gi18n.h>
 
 #include "gobject-utils.h"
-#include "intl.h"
 #include "kazehakase.h"
 #include "kz-icons.h"
 
@@ -33,12 +33,15 @@
 #include "prefs_ui/prefs_lang.h"
 #include "prefs_ui/prefs_font.h"
 #include "prefs_ui/prefs_tab.h"
-#include "prefs_ui/prefs_location_entry.h"
 #include "prefs_ui/prefs_key_accel.h"
 #include "prefs_ui/prefs_gesture.h"
+#include "prefs_ui/prefs_entry.h"
 #include "prefs_ui/prefs_proxy.h"
-#include "prefs_ui/prefs_session.h"
 #include "prefs_ui/prefs_ui_editor.h"
+#include "prefs_ui/prefs_privacy.h"
+#include "prefs_ui/prefs_external_program.h"
+#include "prefs_ui/prefs_history.h"
+
 
 enum {
 	COLUMN_TERMINATOR = -1,
@@ -67,10 +70,14 @@ static void       kz_prefs_win_dispose           (GObject         *object);
 static void       kz_prefs_win_finalize          (GObject         *finalize);
 static void       kz_prefs_win_response          (GtkDialog       *dialog,
 						  gint             arg);
+static gboolean   kz_prefs_win_delete_event      (GtkWidget        *widget,
+						  GdkEventAny      *event);
 
 static GtkWidget *kz_prefs_win_create_nav_tree   (KzPrefsWin *win);
 static void       kz_prefs_win_reset_nav_tree    (KzPrefsWin *win);
 static void       kz_prefs_win_set_ui_level      (KzPrefsWin *win);
+static void       kz_prefs_win_restore_state     (KzPrefsWin *win);
+static void       kz_prefs_win_store_state       (KzPrefsWin *win);
 
 static void       cb_profile_global_changed      (KzProfile   *profile,
 						  const gchar *section,
@@ -111,27 +118,32 @@ kz_prefs_win_class_init (KzPrefsWinClass *klass)
 {
 	GObjectClass *object_class;
 	GtkDialogClass *dialog_class;
+	GtkWidgetClass *widget_class;
 
 	parent_class = g_type_class_peek_parent(klass);
 	object_class = (GObjectClass *) klass;
 	dialog_class = (GtkDialogClass *) klass;
+	widget_class = (GtkWidgetClass *) klass;
 
-	object_class->dispose  = kz_prefs_win_dispose;
-	object_class->finalize = kz_prefs_win_finalize;
-
-	dialog_class->response = kz_prefs_win_response;
+	object_class->dispose      = kz_prefs_win_dispose;
+	object_class->finalize     = kz_prefs_win_finalize;
 
+	dialog_class->response     = kz_prefs_win_response;
+	
+	widget_class->delete_event = kz_prefs_win_delete_event;
+	
 	ADD_PAGE_ENTRY(prefs_general_get_entry);
 	ADD_PAGE_ENTRY(prefs_lang_get_entry);
 	ADD_PAGE_ENTRY(prefs_font_get_entry);
 	ADD_PAGE_ENTRY(prefs_tab_get_entry);
-	ADD_PAGE_ENTRY(prefs_location_entry_get_entry);
+	ADD_PAGE_ENTRY(prefs_entry_get_entry);
 	ADD_PAGE_ENTRY(prefs_key_accel_get_entry);
 	ADD_PAGE_ENTRY(prefs_gesture_get_entry);
-	/* ADD_PAGE_ENTRY(prefs_remote_bookmark_get_entry); */
 	ADD_PAGE_ENTRY(prefs_proxy_get_entry);
-	ADD_PAGE_ENTRY(prefs_session_get_entry);
-	/* ADD_PAGE_ENTRY(prefs_ui_editor_get_entry); */
+	ADD_PAGE_ENTRY(prefs_privacy_get_entry); 
+	ADD_PAGE_ENTRY(prefs_ui_editor_get_entry); 
+	ADD_PAGE_ENTRY(prefs_external_program_get_entry);
+	ADD_PAGE_ENTRY(prefs_history_get_entry);
 }
 
 
@@ -191,10 +203,11 @@ kz_prefs_win_init (KzPrefsWin *win)
 	win->notebook        = notebook;
 
 	/* set default state */
+	kz_prefs_win_restore_state(win);
 	kz_prefs_win_reset_nav_tree(win);
 
 	/* observe UI Level */
-	g_signal_connect(G_OBJECT(kz_global_profile),
+	g_signal_connect(kz_global_profile,
 			 "changed::Global",
 			 G_CALLBACK(cb_profile_global_changed), win);
 }
@@ -205,7 +218,7 @@ kz_prefs_win_dispose (GObject *object)
 	KzPrefsWin *win = KZ_PREFS_WIN(object);
 
 	g_signal_handlers_disconnect_by_func
-		(G_OBJECT(kz_global_profile),
+		(kz_global_profile,
 		 G_CALLBACK(cb_profile_global_changed), win);
 
 	if (win->priv->page_list)
@@ -240,6 +253,7 @@ kz_prefs_win_response (GtkDialog *dialog, gint arg)
 	{
 	case GTK_RESPONSE_ACCEPT:
 	case GTK_RESPONSE_REJECT:
+		kz_prefs_win_store_state(KZ_PREFS_WIN(dialog));
 		gtk_widget_destroy(GTK_WIDGET(dialog));
 		break;
 	case GTK_RESPONSE_APPLY:
@@ -255,6 +269,14 @@ kz_prefs_win_response (GtkDialog *dialog, gint arg)
 	}
 }
 
+static gboolean 
+kz_prefs_win_delete_event (GtkWidget *widget, GdkEventAny *event)
+{
+	kz_prefs_win_store_state(KZ_PREFS_WIN(widget));
+	
+	return FALSE;
+}
+
 GtkWidget *
 kz_prefs_win_new (void)
 {
@@ -366,9 +388,9 @@ kz_prefs_win_create_nav_tree (KzPrefsWin *win)
 	gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree), TRUE);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (tree), FALSE);
 
-	g_signal_connect (G_OBJECT (tree), "key_press_event",
+	g_signal_connect (tree, "key_press_event",
 			  G_CALLBACK (cb_tree_key_press), win);
-	g_signal_connect (G_OBJECT (tree), "cursor_changed",
+	g_signal_connect (tree, "cursor_changed",
 			  G_CALLBACK (cb_tree_cursor_changed), win);
 
 	/* name column */
@@ -432,7 +454,8 @@ kz_prefs_win_create_nav_tree (KzPrefsWin *win)
 				   COLUMN_TERMINATOR);
 		g_free(title);
 	}
-
+	
+	g_object_unref(store);
 	return tree;
 }
 
@@ -518,6 +541,36 @@ kz_prefs_win_reset_nav_tree (KzPrefsWin *win)
 	/* FIXME: set cursor */
 }
 
+static void
+kz_prefs_win_store_state (KzPrefsWin *win)
+{
+	gint client_x, client_y, width, height;
+
+	g_return_if_fail(KZ_IS_PREFS_WIN(win));
+
+	gdk_window_get_geometry (GTK_WIDGET(win)->window,
+				 &client_x, &client_y,
+				 &width, &height, NULL);
+
+	KZ_CONF_SET("PreferenceWindow", "width",  width,  INT);
+	KZ_CONF_SET("PreferenceWindow", "height", height, INT);
+}
+
+
+static void
+kz_prefs_win_restore_state (KzPrefsWin *win)
+{
+	gint width = 600, height = 450;
+
+	g_return_if_fail(KZ_IS_PREFS_WIN(win));
+
+	KZ_CONF_GET("PreferenceWindow", "width",  width, INT);
+	KZ_CONF_GET("PreferenceWindow", "height", height, INT);
+
+	/* size */
+	gtk_window_set_default_size(GTK_WINDOW(win), width, height);
+}
+
 
 GtkWidget *
 kz_prefs_ui_utils_create_title(const gchar *title)
@@ -574,7 +627,7 @@ kz_prefs_win_set_ui_level (KzPrefsWin *win)
 		response = KZ_RESPONSE_UI_LEVEL_BEGINNER;
 		break;
 	}
-	g_signal_emit_by_name(G_OBJECT(win), "response",
+	g_signal_emit_by_name(win, "response",
 			      response);
 }
 
@@ -588,8 +641,6 @@ cb_tree_key_press (GtkWidget *widget, GdkEventKey *event, gpointer data)
 	GtkTreePath *treepath;
 	gboolean success, retval = FALSE;
 
-	g_return_val_if_fail (GTK_IS_TREE_VIEW (widget), FALSE);
-
 	selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (widget));
 	if (!selection) return FALSE;
 	success = gtk_tree_selection_get_selected (selection, &model, &iter);
@@ -635,7 +686,6 @@ cb_tree_cursor_changed (GtkTreeView *treeview, gpointer data)
 	GtkTreeIter iter;
 	KzPrefsWinPagePriv *priv;
 
-	g_return_if_fail(treeview);
 	g_return_if_fail(KZ_IS_PREFS_WIN(data));
 
 	win = KZ_PREFS_WIN(data);
@@ -660,7 +710,6 @@ cb_profile_global_changed (KzProfile *profile,
 			   const gchar *old_value,
 			   KzPrefsWin *win)
 {
-	g_return_if_fail(KZ_IS_PROFILE(profile));
 	g_return_if_fail(KZ_IS_PREFS_WIN(win));
 
 	kz_prefs_win_reset_nav_tree(win);
