@@ -17,7 +17,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: kz-xml.c,v 1.18 2004/03/17 07:23:57 ikezoe Exp $
+ *  $Id: kz-xml.c,v 1.1.1.1 2005/03/27 12:58:21 poincare Exp $
  */
 
 #include "kz-xml.h"
@@ -67,7 +67,7 @@ kz_xml_init (KzXML *xml)
 	xml->dtd           = NULL;
 	xml->encoding      = g_strdup("UTF-8");
 	xml->root          = kz_xml_node_new(xml, KZ_XML_NODE_DOC_ROOT);
-	xml->priv          = g_new0(KzXMLPriv, 1);;
+	xml->priv          = g_new0(KzXMLPriv, 1);
 	xml->priv->context = NULL;
 }
 
@@ -98,7 +98,7 @@ kz_xml_dispose (GObject *object)
 KzXML *
 kz_xml_new (void)
 {
-	KzXML *xml = KZ_XML(g_object_new(KZ_TYPE_XML, NULL));
+	KzXML *xml = g_object_new(KZ_TYPE_XML, NULL);
 	return xml;
 }
 
@@ -278,7 +278,8 @@ kz_xml_load_xml (KzXML *xml, const gchar *buffer, guint length)
 	}
 	else
 	{
-		g_warning("Bookmark parse error!: %s", error->message);
+		g_warning("XML parse error!: %s", error->message);
+		g_error_free(error);
 	}
 	g_markup_parse_context_free (context);
 	xml->priv->context = NULL;
@@ -493,12 +494,13 @@ kz_xml_node_append_xml_string (KzXMLNode *node, GString *gstr)
 	else if (node->type == KZ_XML_NODE_TEXT)
 	{
 		gchar *escaped;
-//		escaped = g_strdup(node->content);
-	
+
 		escaped = g_markup_escape_text(node->content, -1);
 		if (escaped)
+		{
 			g_string_append(gstr, escaped);
-		g_free(escaped);
+			g_free(escaped);
+		}
 	}
 	else
 	{
@@ -908,7 +910,18 @@ kz_xml_node_is_space (KzXMLNode *node)
 	g_return_val_if_fail(node->content, FALSE);
 
 	str = node->content;
-
+	/*
+	 * "" is not space.
+	 * '<title></title>' won't make indent. 
+	 * Now, '<title></title>' does not make indent.
+	 * Formerly, 
+	 * <title></title>
+	 * is converted to 
+	 * <title>
+	 * </title>
+	 */
+	if (strlen(str) == 0)
+		return FALSE;
 	for (c = *str; *str; str++)
 	{
 		if (!isspace(*str))
